

class GutenbergBook:

    def __init__(self, author_name: str = "", author_birth: int = 0, author_death: int = 0, title: str = "",
                 lang: str = "", genre: str = "", subject: str = "", num_chars: int = 0, num_words: int = 0,
                 num_sentences: int = 0, num_difficult_words: int = 0, url: str = "", downloads: int = 0):
            self.author_name = author_name
            self.author_birth = author_birth
            self.author_death = author_death
            self.title = title
            self.lang = lang
            self.genre = genre
            self.subject = subject
            self.num_chars = num_chars
            self.num_words = num_words
            self.num_sentences = num_sentences
            self.num_difficult_words = num_difficult_words
            self.url = url
            self.downloads = downloads

    def get_author_name(self):
        return self.author_name

    def set_author_name(self, author_name):
        self.author_name = author_name

    def get_author_birth(self):
        return self.author_birth

    def set_author_birth(self, author_birth):
        self.author_birth = author_birth

    def get_author_death(self):
        return self.author_death

    def set_author_death(self, author_death):
        self.authorDeath = author_death

    def get_title(self):
        return self.title

    def set_title(self, title):
        self.title = title

    def get_genre(self):
        return self.genre

    def set_genre(self, genre):
        self.genre = genre

    def get_subject(self):
        return self.subject

    def set_subject(self, subject):
        self.subject = subject

    def get_num_words(self):
        return self.num_words

    def set_num_words(self, num_words):
        self.num_words = num_words

    def get_num_sentences(self):
        return self.num_sentences

    def set_num_sentences(self, num_sentences):
        self.num_sentences = num_sentences

    def get_num_difficult_words(self):
        return self.num_difficult_words

    def set_num_difficult_words(self,_num_difficult_words):
        self.num_difficult_words_=_num_difficult_words




from bridges.bst_element import *
##
 #  @brief The This class can be used to create K-d Tree  elements, derived
 #  from BSTElement. K-D trees can be thought of as the spatial equivalent
 #  binary search trees and operate on multiple dimensions (2D and 3D are
 #  most common). These trees serve as a representation of the underlying
 #  geometrically defined spaces. Specialized versions of these trees
 #  include quadtrees and octrees, which subdivide into equal sized quadrants
 #  octants at each level, respectively.
 #  This class extends the BSTElement class by adding a dimension property.
 #  It also includes a thickness property for displaying the partitioning
 #  lines generated by the convex decomposition.
 #
 #  convenient to generate visual representation
 #  to allow for use in a binary search tree implementation.
 #
 # @author Kalpathi Subramanian
 # @date 12/26/18
 #

class KDTreeElement(BSTElement):

    ##
    # Constructs a KdTreeElement wit the provided value, label, key,
    # left and right KdTree elements. The defaults will be used if
    # not provided
    #
    # @param key The key for the ordering
    # @param val The Data to hold
    # @param lab the label to show
    # @param left The left KdTree Element
    # @param right the right KdTree element
    #
    def __init__(self, key = None, dim = None, th = None, left = None, right = None, val = None, lab = None):
        if key is not None and dim is not None and th is not None and left is not None and right is not None and val is not None:
            super(KDTreeElement, self).__init__(e = val, left = left, right = right)
            self.thickness = th
            self.dimension = dim
            self.set_key(key)
        elif key is not None and val is not None and left is not None and right is not None:
            super(KDTreeElement, self).__init__(e = val, left=left, right=right)
            self.thickness = 0.0
            self.dimension = 0
            self.set_key(key)
        elif val is not None and left is not None and right is not None:
            super(KDTreeElement, self).__init__(e = val, left=left, right=right)
            self.thickness = 0.0
            self.dimension = 0
        elif left is not None and right is not None:
            super(KDTreeElement, self).__init__(left=left, right=right)
        elif key is not None and dim is not None:
            super(KDTreeElement, self).__init__()
            self.dimension = dim
            self.thickness = 0.0
            self.set_key(key)
        elif key is not None and val is not None:
            super(KDTreeElement, self).__init__(e = val)
            self.set_key(key)
        elif key is not None and val is not None and lab is not None:
            super(KDTreeElement, self).__init__(e = val, label = lab)
            self.set_key(key)
        elif lab is not None and val is not None:
            super(KDTreeElement, self).__init__(e = val, label = lab)
        elif val is not None:
            super(KDTreeElement, self).__init__(e = val)
        else:
            super(KDTreeElement, self).__init__()
            self.thickness = 0.0
            self.dimension = 0


    def get_data_structure_type(self):
        return "KDTree"

    def get_dimension(self):
        return self.dimension

    def set_dimension(self, dim):
        self.dimension = dim

    def get_thickness(self):
        return self.thickness

    def set_thickness(self, th):
        self.thickness = th

    def get_left(self):
        return super(KDTreeElement, self).get_left()

    def get_right(self):
        return super(KDTreeElement, self).get_right()




import time


class EarthquakeUSGS:

    def getDate(self):

        s = self.get_time()
        epoch_time = int(self.get_time())
        eq_time = epoch_time/1000

        eqt = time.gmtime(eq_time)

        year = eqt.tm_year + 1900
        month = eqt.tm_mon
        day = eqt.tm_mday
        hour = eqt.tm_hour
        sec = eqt.tm_sec
        min = eqt.tm_min

    def __init__(self, magnitude = None, longit = None, latit = None, location = None, title = None, url = None, time = None):
        if magnitude is not None:
            self.magnitude = magnitude
        else:
            self.magnitude = 0.0

        if longit is not None:
            self.longit = longit
        else:
            self.longit = 0.0

        if latit is not None:
            self.latit = latit
        else:
            self.latit = 0.0

        if location is not None:
            self.location = location
        else:
            self.location = ""

        if title is not None:
            self.title = title
        else:
            self.title = ""

        if url is not None:
            self.url = url
        else:
            self.url = ""

        if time is not None:
            self.time = time
        else:
            self.time = ""
            self.year = self.month = self.day = self.hour = self.min = self.sec =0

    def get_time(self):
        return self.time

    def set_time(self, tm):
        self.time = tm

    def get_year(self):
        self.getDate()
        return self.year

    def get_month(self):
        self.getDate()
        return self.month

    def get_day(self):
        self.getDate()
        return self.day

    def get_hour(self):
        self.getDate()
        return self.hour

    def get_minutes(self):
        self.getDate()
        return self.min

    def get_seconds(self):
        self.getDate()
        return self.sec

    def get_latit(self):
        return self.latit

    def set_latit(self, latit):
        self.latit = latit

    def get_longit(self):
        return self.longit

    def set_longit(self, longit):
        self.longit = longit

    def get_location(self):
        return self.location

    def set_location(self, location):
        self.location = location

    def get_title(self):
        return self.title

    def set_title(self, title):
        self.title = title

    def get_url(self):
        return self.url

    def set_url(self, url):
        self.url = url

    def get_magnitude(self):
        return self.magnitude

    def set_magnitude(self,magnitude):
        self.magnitude = magnitude








class Game:

    def __init__(self, title: str = "", platform: str = "", rating: float = 0.0, genre: str = ""):
            self.title = title
            self.platform = platform
            self.rating = rating
            self.genre = genre

    def get_title(self):
        return self.title

    def set_title(self, title):
        self.title = title

    def get_platform(self):
        return self.platform

    def set_platform(self, platform):
        self.platform = platform

    def get_rating(self):
        return self.rating

    def set_rating(self, rating):
        self.rating = rating

    def get_genre(self):
        return self.genre

    def set_genre(self, genre):
        self.genre = genre



##
#  @brief This is a class in BRIDGES for representing an (n x n) grid.
#  @author David Burlinson
#  @param
import traceback

class Grid:
    grid = []
    gridSize = [10, 10]
    maxGridSize = [1080, 1920]

    def get_data_structure_type(self):
        return "Grid"

    ##
    # Grid constructors
    # @param size - size of the grid as array
    # @param rows - number of rows in grid
    # @param cols - number of columns in grid
    #
    def __init__(self, size = None, rows = None, cols = None):
        if size is not None:
            if ((size[0] <= 0 or size[0] > self.maxGridSize[0]) or (size[1] <= 0 or size[1] > self.maxGridSize[1])):
                raise ValueError("Invalid size: [" + str(str(size[0]) + "," + str(size[1]) + "] .. please use values between (0 and " + str(self.maxGridSize[0]) + "] for rows and values between (0 and " + str(self.maxGridSize[1]) + "] for columns"))
            self.grid = []
            for i in range(size[0]):
                self.grid.append([])
                for j in range(size[1]):
                    self.grid[i].append(None)
        if rows is not None and cols is not None:
            size = [rows, cols]
            if ((size[0] <= 0 or size[0] > self.maxGridSize[0]) or (size[1] <= 0 or size[1] > self.maxGridSize[1])):
                raise ValueError("Invalid size: [" + str(str(size[0]) + "," + str(size[1]) + "] .. please use values between (0 and " + str(self.maxGridSize[0]) + "] for rows and values between (0 and " + str(self.maxGridSize[1]) + "] for columns"))
            self.grid = []
            for i in range(size[0]):
                self.grid.append([])
                for j in range(size[1]):
                    self.grid[i].append(None)

    ##
    #  set up inner lists (columns)
    #  initialize values in inner lists
    def get_dimensions(self):
        return [self.gridSize[0], self.gridSize[1]]

    ##
    #  get the (row, col) element in the grid
    def get(self, row, col):
        try:
            return self.grid[row][col]
        except Exception as e:
            traceback.print_tb(e.__traceback__)
            return None

    ##
    #  set the (row, col) element in the grid
    def set(self, row, col, val):
        self.grid[int(row)][int(col)] = val



import json
from bridges.symbol import *

class Polygon(Symbol):

    def __init__(self, pts = None):
        super(Polygon, self).__init__()
        if pts is not None:
            self.points = pts
        else:
            self.points = []
        self.shape = "polygon"

    def get_name(self):
        return "polygon"

    def add_point(self, x, y):
        fx = float(x)
        fy = float(y)

        if(fx > float('-inf') and fx < float('inf') and fy > float('-inf') and fy < float('inf')):
            self.points.append(fx)
            self.points.append(fy)

    def get_points(self):
        return self.points

    def set_polygon(self, pts):
        self.points = pts

    def get_dimensions(self):
        minx = float('inf')
        miny = float('inf')
        maxx = float('-inf')
        maxy = float('-inf')

        x = 0.0
        y = 0.0
        for i in range(0,len(self.points),2):
            x = self.points[i]
            y = self.points[i+1]
            if (x < minx):
                minx = x
            if (x > maxx):
                maxx = x
            if (y < miny):
                miny = y
            if (y > maxy):
                maxy = y

        return [minx, maxx, miny, maxy]

    def get_json_representation(self):

        ds_json = super(Polygon, self).get_json_representation()
        ds_json["name"] = self.get_label()
        ds_json["shape"] = self.get_name()
        ds_json["points"] = self.points

        return ds_json


#!/usr/bin/env python
from decimal import Decimal
from bridges.element import *
from bridges.color import *
from decimal import Decimal


##
# This class is used to store the visualization elements on the for the bridges
# Visualiztion, including the color, shape, opacity, and size of the node.
# <p>
# Objects of this class are stored as properties of all Element subclasses.
# Generally, you will manipulating the ElementVisualizer returned from the
# Element getVisualizer() method, and then call the setVisualizer() method on
# the Element after changes have been made.
#
class ElementVisualizer(object):
    #  Visualization properties for this Node.

    shape = "circle"
    key = ""
    locationX = Decimal("Infinity")
    locationY = Decimal("Infinity")
    size = 10.0
    opacity = 1.0

    prop = dict()

    prop["color"] = "[70, 130, 180, 1.0]"
    prop["opacity"] = "1.0"
    prop["size"] = "10.0"
    prop["shape"] = "circle"
    prop["key"] = ""
    prop["locationX"] = str(locationX)
    prop["locationY"] = str(locationY)

    ##
    # Construct an ElementVisualizer with the default visualization settings.
    # The default settings are color = green, opacity = 1.0, size = 10.0, shape
    # = circle.
    #
    def __init__(self, a_color="green", a_shape ="circle", size=10.0, opacity=1.0):
        if a_color is not "green":
            self.set_color(a_color)
        else:
            self.color = Color(70, 130, 180, 1.0)
        if a_shape is not "circle":
            self.set_shape(a_shape)
        if size is not 10.0:
            self.set_size(10.0)
        if opacity is not 1.0:
            self.set_opacity(opacity)

    ##
    # Set the size of the Element in the Bridge Visualization in pixels
    #
    # @param size
    #            the pixel size of the Element in the bridges Visualization
    #
    def set_size(self, size):
        self.prop["size"] = str(size)
        self.size = size

    ##
    # Get the size of the Element in the bridges Visualiation
    #
    # @return the size in pixels of the Element in the bridges Visualization
    #
    def get_size(self):
        return self.size

    ##
    #  Set the color of the Element in the bridges Visualization to "aColor".
    #  @param aColor the string reprsenting the color of the Element in the bridges Visualization
    #
    def set_color(self, *args, **kwargs):
        """
        Usage: requires either 3 ints 0-255 for RGB and an optional float 0.0-1.0 for alpha or a str of a web color
        can also key the RGBA values with r, g, b, a or red, green, blue, alpha respectively and col_name for the str
        :param args: int, int, int optional float or str
        :param kwargs: r/red: int, b/blue: int, g/green: int optional a/alpha: float or col_name: str
        :return: None
        """
        self.color.set_color(*args, **kwargs)

    ##
    # Get the color of the Element in the bridges Visualization
    #  @return the string reprsenting the color of the Element in the bridges Visualization
    #
    def get_color(self):
        return self.color

    ##
    # Get the shape of the Element in the bridges Visualization.
    # @return the string that represents the Element's shape in the bridges Visualization.
    #
    def get_shape(self):
        return self.shape

    ##
    # Sets the shape of the Element in the bridges Visualization
    #
    # @param aShape the string representing the shape of the Element in the bridges Visualization
    #
    def set_shape(self, a_shape):
        #  this.aShape = aShape;
        a_shape = a_shape.lower()
        self.prop["shape"] = a_shape
        self.shape = a_shape

    ##
    # Sets the opacity of the Element in the bridges Visualization
    #
    # @param opacity a double between 0 and 1 representing how transparent the node
    #            should be on the bridges Visualization. 0 for invisible, 1 for
    #            fully visible, a decimal between 0 and 1 for varying
    #            transparency.
    #
    def set_opacity(self, opacity):
        self.prop["opacity"] = Decimal(opacity)
        self.color.set_alpha(opacity)
        self.opacity = opacity

    ##
    #  Get the opacity of the Element in the bridges Visualization
    # @return the opacity value
    #
    def get_opacity(self):
        return self.color.get_alpha()

    ##
    # The randomColor method selects a random color from the available list of
    # colors found in Validation.java and sets the color of the current element
    #
    # @return a color name as a string value
    #
    # def random_color(self):
    #     a = Validation.COLOR_NAMES.toArray()
    #     return self.setColor(a[Random().nextInt(a.length)].__str__())

    def set_location(self, x, y):
        self.locationX = x
        self.locationY = y

    def get_locationX(self):
        return self.locationX

    def get_locationY(self):
        return self.locationY



#!/usr/bin/env python
from bridges.element import *
from bridges.sl_element import *
from bridges.edge import *
import traceback
import sys
import json

##
#
#	@brief The GraphAdjList class can be used to represent adjacency list based
#		graphs in BRIDGES
#
#	The GraphAdjList class can be used to represent adjacency list based  graphs
#	in BRIDGES; it takes 2 generic parameters: (1) K, which is an orderable
#	key value used in accessing vertices (in constant time) using a hashmap. This
#	permits data sets that need to be accessed by keys that are strings, and
#	(2) E, an application defined type, and used in the Edge representation.
#	The class is simply a wrapper  around the Java Hashmap class
#	and, thus, derives all its operations from it.
#	BRIDGES provides methods to visualize the graph  and its contents.
#
#	The vertices of the graph are held in a Java hashmap, for near constant time access;
#	this lets us use strings or integral ids for vertices. The adjacency lists,
#	also a Java hashmap  are built for each vertex and contain the edge (terminating
#	vertex id, weight) in the Edge structure, defined separately. Adjacency lists
#	are singly linked lists using the BRIDGES SLelement.
#
#	Convenience methods are provided to add vertices and edges to the graph as well as
#	retrieve the adjacency list of a vertex, given its id.
#
#
class GraphAdjList():

    QUOTE = "\""
    COMMA = ","
    COLON = ":"
    OPEN_CURLY = "{"
    CLOSE_CURLY = "}"
    OPEN_PAREN = "("
    CLOSE_PAREN = ")"
    OPEN_BOX = "["
    CLOSE_BOX = "]"

    LargeGraphVertSize = 1000;
    ##
    #
    # 	Constructor
    #
    def __init__(self):
        self.vertices = dict()
        self.adj_list = dict()

    ##
    # 	This method gets the data structure type
    #
    # 	@return  The date structure type as a string
    #
    #
    def get_data_structure_type(self):
        if self.LargeGraphVertSize < len(self.vertices):
            return "largegraph"
        return "GraphAdjacencyList"

    ##
    # Adds a new vertex to the graph, initializes the adjacency
    # list; user is responsible for checking if the vertex already
    # exists. This method will replace the value for this key
    #
    #	@param k - vertex id
    #	@param E - vertex info, currently used as a label by default
    #
    def add_vertex(self, k, e):
        #  note: it is the user's responsibility to  check
        #  for duplicate vertices
        self.vertices[k] = Element(val = e)
        self.vertices.get(k).set_label(str(k))
        self.adj_list[k] = None

    ##
    #	Adds a new edge to the graph, adds it to that vertex's
    #	adjacency list; user is responsible for checking if the
    #	vertex already exists. This version assumes a default edge
    # 	weight of 1.
    #
    #	@param src - source vertex of edge
    #	@param dest - destination  vertex of edge
    #
    #
    def add_edge(self, src, dest, weight = None):
        #  check to see if the two vertices exist, else
        #  throw an exception
        try:
            if self.vertices.get(src) is None or self.vertices.get(dest) is None:
                raise ValueError("Vertex " + src + " or " + dest + " does not exist! Add the vertex before creating the edge.")
        except Exception as e:
            traceback.print_tb(e.__traceback__)
        if weight is not None:
            self.adj_list[src] = SLelement(e=Edge(weight, dest), next = self.adj_list.get(src))
        else:
            self.adj_list[src] = SLelement(e=Edge(1, dest), next = self.adj_list.get(src))

    ##
    #	This method returns the graph nodes
    #
    #	return - vertices held in  in the hashmap
    #
    #
    def get_vertices(self):
        return self.vertices

    ##
    #	This is a convenience method to retrieve a vertex given
    #	its key
    #
    #   @param key - The key for the value in vertices dict
    #	@return - graph vertex corresponding to its key
    #
    #
    def get_vertex(self, key):
        return self.vertices.get(key)

    ##
    #	Gets the adjacency list (of type SLelement<Edge> )
    #
    #   @param vertex - The vertex in adj_list
    #	@return - the graph's adjacency lists
    #
    #
    def get_adjacency_list(self, vertex = None):
        if vertex is not None:
            return self.adj_list.get(vertex)
        else:
            return self.adj_list

    ##
    #
    #	 This is a convenience method to simplify access to the link visualizer;
    #	 the method assumes the vertex names point to existing vertices, else an exception
    #	 is thrown
    #
    #   @param src - source vertex of edge
    #   @param dest - destination vertex of edge
    #
    def get_link_visualizer(self, src, dest):
        #  get the source and destination vertex elements
        #  and check to see if they exist
        v1 = self.vertices.get(src)
        v2 = self.vertices.get(dest)
        try:
            if v1 is None or v2 is None:
                raise ValueError("Vertex " + src + " or " + dest + " does not exist! First add the vertices to the graph.")
        except Exception as e:
            traceback.print_tb(e.__traceback__)
        return v1.get_link_visualizer(v2)

    ##
    #
    #	This is a convenience method to simplify access to the element visualizer;
    #	the method assumes the vertex name points to an existing vertice, else an
    #	exception is thrown
    #
    #   @param vertex - The vertex for which visualizer is wanted
    #
    def get_visualizer(self, vertex):
        #  get the source and destination vertex elements
        #  and check to see if they exist
        v = self.vertices.get(vertex)
        try:
            if v is None:
                raise ValueError("Vertex " + vertex + " does not exist! First add the vertices to the graph.")
        except Exception as e:
            traceback.print_tb(e.__traceback__)
        return v.get_visualizer()

    ##
    #	Get the JSON representation of the the data structure
    #
    def get_data_structure_representation(self) -> str:

        # redirect for large graphs
        if len(self.vertices) > self.LargeGraphVertSize:
            return self._get_data_structure_large_graph()

        #  map to reorder the nodes for building JSON
        node_map = dict()

        #  get the list nodes
        nodes = []

        # get the objects and add them to the array
        for elements in self.vertices.items():
            nodes.append(elements[1])

        #  remap  map these nodes to  0...MaxNodes-1
        #  and build the nodes JSON
        nodes_JSON = str()

        k = 0
        while k < len(nodes):
            node_map[nodes[k]] = k
            nodes_JSON += (nodes[k].get_element_representation())# get the element object and represent it
            nodes_JSON += self.COMMA
            k += 1
        #  remove the last comma
        if len(nodes) != 0:
            nodes_JSON = nodes_JSON[:-1]
        #  build the links JSON - traverse the adj. lists
        links_JSON = str()
        for a_list in self.adj_list.items():
            list = a_list[1]
            src_vert = self.vertices.get(a_list[0])
            #  get the source vertex index for the JSON (int)
            while list is not None:
                src_indx = node_map.get(src_vert)
                #  get the destination vertex index for the JSON (int)
                edge = list.get_value()
                dest_vert = self.vertices.get(edge.get_vertex())
                dest_indx = node_map.get(dest_vert)
                #  get link representation
                links_JSON+=(list.get_link_representation(src_vert.get_link_visualizer(dest_vert), str(src_indx), str(dest_indx)))
                links_JSON+=(self.COMMA)
                list = list.get_next()
        #  remove the last comma
        if len(links_JSON) > 0:
            links_JSON = links_JSON[:-1]
        json_str = self.QUOTE + "nodes" + self.QUOTE + self.COLON + self.OPEN_BOX + nodes_JSON.__str__() + self.CLOSE_BOX + self.COMMA + self.QUOTE + "links" + self.QUOTE + self.COLON + self.OPEN_BOX + links_JSON.__str__() + self.CLOSE_BOX + self.CLOSE_CURLY
        return json_str

    def _get_data_structure_large_graph(self) -> str:
        nodes_data = []
        node_map = {}
        for i, (key, element) in enumerate(self.vertices.items()):
            node_map[key] = i
            vis = element.get_visualizer()
            this_node_data = []
            if vis.locationX != Decimal("Infinity") and vis.locationY != Decimal("Infinity"):
                this_node_data.append([vis.locationX, vis.locationY])

            color = vis.color
            this_node_data.append([x for x in color.rgba])
            nodes_data.append(this_node_data)

        links_data = []
        for key, element in self.vertices.items():
            adj_ele = self.adj_list.get(key)
            while adj_ele is not None:
                src = node_map[key]
                dest = node_map[adj_ele.get_value().get_vertex()]
                color = element.get_link_visualizer(self.get_vertex(adj_ele.get_value().get_vertex())).color

                links_data.append([src, dest, [x for x in color.rgba]])
                adj_ele = adj_ele.next

        wrapper = {
            "nodes": nodes_data,
            "links": links_data,
        }
        ret = json.dumps(wrapper)
        ret = ret[1:]
        return ret




from bridges.dl_element import *
from bridges.bridges import *
from StudentInfo import *
import sys
import os



def insertFront(front, new_el):
    if front is None:
        return new_el

    new_el.set_next(front)
    front.set_prev(new_el)

    return new_el

def main():

    args = sys.argv[1:]

    # create the Bridges object, set credentials
    bridges = Bridges(1000, os.environ.get("BRIDGES_USER_NAME"), os.environ.get("BRIDGES_API_KEY"))

    if len(args) > 3:
        bridges.connector.set_server(args[3])

    students = []

    # load student info
    students.append(Student("00000000000",
                              "Gretel Chaney",
                              "CS",
                              "g.chaney@generated.com",
                              "magenta",
                              "blue",
                              9.0))

    students.append(Student("00000000001",
                              "Karol Soderman",
                              "SIS",
                              "k.soderman@generated.com",
                              "magenta",
                              "red",
                              11.0))

    students.append(Student("00000000002",
                              "Lamont Kyler",
                              "BIO",
                              "l.kyler@generated.com",
                              "yellow",
                              "green",
                              50.0))

    students.append(Student("00000000003",
                              "Gladys Serino",
                              "CS",
                              "g.serino@generated.com",
                              "green",
                              "magenta",
                              9.0))

    students.append(Student("00000000004",
                              "Starr Mcginn",
                              "CS",
                              "s.mcginn@generated.com",
                              "red",
                              "cyan",
                              15.0))

    # insert the students in front of the list
    head = None
    for i in range(len(students)):
        head = insertFront(head, DLelement(label = "", e = students[i]))

    # add visual attributes
    curr = head
    while curr is not None:
        curr.set_label(curr.get_value().getName())
        curr.get_visualizer().set_color(col_name=curr.get_value().getDislikeColor())

        if curr.get_next()is not None:
            next = curr.get_next()
            curr.get_link_visualizer(next).set_color(col_name=curr.get_value().getDislikeColor())
            next.get_link_visualizer(curr).set_color(col_name=curr.get_value().getDislikeColor())

        curr = curr.get_next()


    # set dat structure to be visualized
    bridges.set_data_structure(head)
    # visualize the data structure
    bridges.visualize()

if __name__ == "__main__":
    main()



#!/usr/bin/env python
from bridges.element import *

##
# 	@brief This class can be used to instantiate Singly Linked Elements.
# 	This class extends Element and takes a generic parameter <E>
#	representing application specific data. This element forms the basic
#	building block for singly linked lists. Singly linked elements have a field
#	pointing to the next element along the list.
#
#
# 	Elements contain a visualizer (ElementVisualizer) object for setting visual
#	attributes (color, shape, opacity, size), necessary for displaying them in a
#	web browser.
#
#	Elements also have a LinkVisualizer object, that is used when they are linked to
#	another element, appropriate for setting link attributes, for instance, between
#	the current element and its next element.
#
#
class SLelement(Element):

    ##
    # This constructor creates an SLelement object
    # and sets the next pointer to null
    # @param label - the label of SLelement that shows up on the bridges visualization
    # @param e  - the generic object that this SLelement will hold
    # @param next - the element that should be assigned to the next pointer
    # 
    def __init__(self, e = None, label = None, next = None):
        if e is not None and label is not None:
            super(SLelement, self).__init__(val = e, label = label)
            self.next = None
        if e is not None and label is None and next is not None:
            super(SLelement, self).__init__(val = e)
            self.set_next(next)
        if e is not None and label is None and next is None:
            super(SLelement, self).__init__(val = e)
            self.next = None
        if e is None and label is None and next is not None:
            self.set_next(next)
        else:
            super(SLelement, self).__init__()
            self.next = next


    ##
    #
    #	This method gets the data structure type
    #
    #	@return  The date structure type as a string
    #
    #
    def get_data_structure_type(self):
        return "SinglyLinkedList"

    ##
    # Retrieves the element following this element
    #
    # @return - SLelement assigned to next
    #
    #
    def get_next(self):
        return self.next

    ##
    # Retrieves the value in the SLelement
    # @return - SLElement value held
    def get_value(self):
        return super(SLelement, self).get_value()
    ##
    # Sets the element to point to the next SLelement
    #
    # @param next - SLelement that should be assigned to the next pointer
    #
    def set_next(self, next):
        self.next = next
        if next is not None:
            self.set_link_visualizer(next)


    def __str__(self):
        return "SLelement [next=" + self.next + ", getNext()=" + self.get_next() + ", getIdentifier()=" + self.get_identifier() + ", getVisualizer()=" + self.get_visualizer() + ", getClassName()=" + self.get_class_name() + ", getElementRepresentation()=" + self.get_element_representation() + ", getLabel()=" + self.get_label() + ", getValue()=" + self.get_value() + ", toString()=" + super(SLelement, self).__str__() + ", getClass()=" + self.getClass() + ", hashCode()=" + self.hashCode() + "]"

    ##
    #	Get the JSON representation of the the data structure
    #
    def get_data_structure_representation(self):
        #  map to reorder the nodes for building JSON
        node_map = dict()
        #  get teh list nodes
        nodes = []
        self.get_list_elements(nodes)
        #  generate the JSON of the list nodes
        nodes_JSON = str()
        k = 0
        while k < len(nodes):
            node_map[nodes[k]] = k
            nodes_JSON += nodes[k].get_element_representation()
            nodes_JSON += self.COMMA
            k += 1
        #  remove the last comma
        if len(nodes) != 0:
            nodes_JSON = nodes_JSON[:-1]
        links_JSON = str()
        k = 0
        while k < len(nodes):
            par = nodes[k]
            chld = par.next
            if chld is not None:
                #  add the link
                links_JSON += (self.get_link_representation(par.get_link_visualizer(chld), str(node_map.get(par)), str(node_map.get(chld))))
                links_JSON += (self.COMMA)
            k += 1
        #  add the link
        if len(links_JSON) > 0:
            links_JSON = links_JSON[:-1]
        json_str = self.QUOTE + "nodes" + self.QUOTE + self.COLON + self.OPEN_BOX + str(nodes_JSON) + self.CLOSE_BOX + self.COMMA + self.QUOTE + "links" + self.QUOTE + self.COLON + self.OPEN_BOX + str(links_JSON) + self.CLOSE_BOX + self.CLOSE_CURLY
        return json_str

    ##
    #	Get the elements of the list
    #
    #	@param nodes  a vector of the ndoes in the list
    #
    #
    def get_list_elements(self, nodes):
        el = self
        #  try to handld all lists in subclasses, except multilists
        nodes.clear()
        while el is not None:
            nodes.append(el)
            el = el.get_next()
            #  handle circular lists
            if el == self:
                break




class Song:

    def __init__(self, artist: str = "", song: str = "", album: str = "", lyrics: str = "", release_date: str = ""):
        self.artist = artist
        self.song = song
        self.lyrics = lyrics
        self.album = album
        self.release_date = release_date

    def get_artist(self):
        return self.artist

    def set_artist(self, artist):
        self.artist = artist

    def get_song_title(self):
        return self.song

    def set_song_title(self, song):
        self.song = song

    def get_album_title(self):
        return self.album

    def set_album_title(self, album):
        self.album = album

    def get_lyrics(self):
        return self.lyrics

    def set_lyrics(self, lyrics):
        self.lyrics = lyrics

    def get_release_date(self):
        return self.release_date

    def set_release_date(self, date):
        self.release_date = date



#!/usr/bin/env python
from bridges.dl_element import *
##
#  @brief This class can be used to instantiate Circular Doubly Linked
#  List Elements.
#
#	Structurally they are the same as doubly linked elements
#  except that each node constructed with the next and the previous pointers
#	points to itself.
#
#  User's implementation of the circularly linked list needs to ensure that
#  the last node's next pointer points to the first node and the first node's
#	previous pointer points to the last node, as the visualization generation
#	is dependent on this.
#
#  Elements have labels (string) that are displayed on the visualization.
#  Elements take an generic object E as a user defined parameter, which can be
#	any native type or object.
#
#  Elements contain a visualizer (ElementVisualizer) object for setting visual
#	attributes (color, shape, opacity, size), necessary for displaying them in a web
#	browser.
#
#	Elements also have a LinkVisualizer object that is used when they are
#	linked to another element, appropriate for setting link attributes, between
#  the element and its previous or next nodes.
#
#  @author Kalpathi Subramanian
#
#	@date   7/17/16, 1/16/17
#
#
class CircDLelement(DLelement):

    ##
    #
    #	Constructs an empty CircDLelement with next and prev pointers set
    #	to itself
    # @param label the label for this CircDLelement
    # @param e the genereic object that this CircDLelement is holding
    # @param next the DLelement that should be assigned to the next pointer
    # @param prev the DLelement that should be assigned to the prev pointer
    #
    def __init__(self, e = None, label = None, next = None, prev = None):
        if e is not None and label is not None:
            super(CircDLelement, self).__init__(e = e, label = label)
            self.set_next(self)
            self.set_prev(self)
        elif e is not None and next is not None and prev is not None:
            super(CircDLelement, self).__init__(e = e, next = next, prev = prev)
        elif e is None and next is not None and prev is not None:
            super(CircDLelement, self).__init__()
            self.set_next(next)
            self.set_prev(prev)
        else:
            super(CircDLelement, self).__init__()
            self.next = self
            self.prev = self


    ##
    #
    #	This method gets the data structure type
    #
    #	@return  The date structure type as a string
    #
    #
    def get_data_structure_type(self):
        return "CircularDoublyLinkedList"

    ##
    #
    # This method returns the pointer to the next DLelement
    #
    # @return the DLelement assigned to the next pointer
    #
    #
    def get_next(self):
        return self.next

    ##
    #
    # This method returns the pointer to the previous DLelement
    #
    # @return the DLelement assigned to the prev pointer
    #
    #
    def get_prev(self):
        return self.prev



from bridges.color import *
from bridges.grid import *
import base64


##
#  @brief This is a class in BRIDGES for representing an (n x n) grid.
#  @author David Burlinson
#
#
class ColorGrid(Grid):
    QUOTE = "\""
    COMMA = ","
    COLON = ":"
    OPEN_CURLY = "{"
    CLOSE_CURLY = "}"
    OPEN_PAREN = "("
    CLOSE_PAREN = ")"
    OPEN_BOX = "["
    CLOSE_BOX = "]"

    baseColor = Color(r=0, g=0, b=0, a=1.0)

    def get_data_structure_type(self):
        return "ColorGrid"

    #
    #  Grid constructor with size arguments
    #
    #  @param rows - int representing the number of rows of the grid
    #  @param cols - int representing the number of columns of the grid
    #  @param color - Color object
    #
    def __init__(self, rows: int = 10, cols: int = 10, color: Color = baseColor):
        """
        Grid constructor
        :param int rows: number of rows in the grid
        :param int cols: number of columns in the grid
        :param Color color: base color of the each grid pixel
        """
        super(ColorGrid, self).__init__(rows=rows, cols=cols)
        self.baseColor = color
        self.gridSize = [rows, cols]

        self.initialize_grid()

   ##
   #  Populate the grid with the base color
   #
   #
    def initialize_grid(self):
        for i in range(self.gridSize[0]):
            for j in range(self.gridSize[1]):
                self.set(i, j, self.baseColor)

    ##
    # set the (row, col) element in the ColorGrid
    # @param row - which row to access
    # @param col - which col to access
    # @param color - background color for the cell at row,col
    def set(self, row, col, color):
        super(ColorGrid, self).set(row, col, color)

    ##
    # get the Run Length Encoding of ColorGrid
    #
    def get_rle(self):

        img_bytes = bytearray()
        count = 0
        total_count = 0
        pos = 0
        last = self.grid[0][0]

        while pos < self.gridSize[0] * self.gridSize[1]:
            posY = pos / self.gridSize[1]
            posX = pos % self.gridSize[1]
            current = self.grid[int(posY)][int(posX)]

            if count == 0:
                count = 1
                last = current
            else:
                if last == current:
                    count += 1
                else:
                    total_count += count
                    img_bytes.append(count-1)
                    last = last.get_byte_representation()

                    for k in range(len(last)):
                        img_bytes.append(last[k])

                    count = 1
                    last = current
            if count == 256:
                total_count += count
                img_bytes.append(count-1)
                last = last.get_byte_representation()
                for k in range(len(last)):
                    img_bytes.append(last[k])
                count = 0
            pos += 1
        total_count += count
        img_bytes.append(count-1)
        last = last.get_byte_representation()
        for k in range(len(last)):
            img_bytes.append(last[k])

        if total_count != self.gridSize[0] * self.gridSize[1]:
            print("Something broke in getRLE construction")

        return img_bytes

    ##
    # get raw encoding of ColorGrid
    def get_raw(self):
        img_bytes = bytearray()
        for i in range(self.gridSize[0]):
            if self.grid[i] is not None:
                for j in range(self.gridSize[1]):
                    if self.grid[i][j] is not None:
                        color = self.grid[i][j]
                        color = color.get_byte_representation()
                        for k in range(len(color)):
                            img_bytes.append(color[k])
        return img_bytes

    ##
    #  
    #   get the JSON representation of the color grid
    #
    #   @return the JSON representation of the color grid
    #
    def get_data_structure_representation(self):
        # imageBytes = bytearray()
        # for i in range(self.gridSize[0]):
        #     if (self.grid[i] is not None):
        #         for j in range(self.gridSize[1]):
        #             color = self.grid[i][j]
        #             color = color.getByteRepresentation()
        #             for k in range(len(color)):
        #                 imageBytes.append(color[k])
                    # for k in range(len(color)):
                    #     imageBytes.append(int.from_bytes(color[k], 'little'))
        byte_buff = self.get_rle()
        encoding = "RLE"

        if len(byte_buff) > self.gridSize[0] * self.gridSize[1] * 4:
            encoding = "RAW"
            byte_buff = self.get_raw()
            print("RAW ran")
        else:
            print("RLE ran")


        # json_str = self.QUOTE + "nodes" + self.QUOTE + self.COLON + self.OPEN_BOX + self.QUOTE + base64.b64encode((imageBytes)).decode() + self.QUOTE + self.CLOSE_BOX + self.COMMA
        json_str = self.QUOTE + "encoding" + self.QUOTE + self.COLON + self.QUOTE + encoding + self.QUOTE + self.COMMA + self.QUOTE + "nodes" + self.QUOTE + self.COLON + self.OPEN_BOX  + self.QUOTE + base64.b64encode(bytes(byte_buff)).decode() + self.QUOTE + self.CLOSE_BOX + self.COMMA

        json_str += self.QUOTE + "dimensions" + self.QUOTE + self.COLON + self.OPEN_BOX + str(self.gridSize[0]) + "," + str(self.gridSize[1]) + self.CLOSE_BOX + self.CLOSE_CURLY

        return json_str



import math
from bridges.graph_adj_list import *


class OsmEdge:
    """OSM edge, represents edge between OsmVertex points
    """
    @property
    def source(self) -> int:
        """ID of source vertex
        :return: int
        """
        return self._source

    @source.setter
    def source(self, source: int):
        try:
            value = int(source)
        except ValueError:
            raise ValueError("Source must be an int")

        self._source = value

    @source.deleter
    def source(self):
        del self._source

    @property
    def destination(self) -> int:
        """ID of destination vertex
        :return: int
        """
        return self._destination

    @destination.setter
    def destination(self, destination: int):
        try:
            value = int(destination)
        except ValueError:
            raise ValueError("Destination must be an int")

        self._destination = value

    @destination.deleter
    def destination(self):
        del self._destination

    @property
    def distance(self) -> float:
        """Distance between two vertices
        :return: float
        """
        return self._distance

    @distance.setter
    def distance(self, distance: float):
        try:
            value = float(distance)
        except ValueError:
            raise ValueError("Distance must be a float")

        self._distance = value

    @distance.deleter
    def distance(self):
        del self._distance


    def __init__(self, source: int, destination: int, distance: float):
        """OsmEdge, represents edge between two OsmVertex points
        :param source: int, ID of source vertex
        :param destination: ID of destination vertex
        :param distance: float, distance between the vertices
        """
        self._source = 0
        self._destination = 0
        self._distance = 0.0
        self.source = source
        self.destination = destination
        self.distance = distance


class OsmVertex:
    """OSM vertex, represents vertex using open street map data
    """

    @property
    def latitude(self) -> float:
        """Latitude of vertex
        :return: float
        """
        return self._latitude

    @latitude.setter
    def latitude(self, latitude: float):
        try:
            value = float(latitude)
        except ValueError:
            raise ValueError("latitude must be a float")

        self._latitude = value
        self._to_cartesian_coord()

    @latitude.deleter
    def latitude(self):
        del self._latitude

    @property
    def longitude(self) -> float:
        """Longitude of vertex
        :return: float
        """
        return self._longitude

    @longitude.setter
    def longitude(self, longitude: float):
        try:
            value = float(longitude)
        except ValueError:
            raise ValueError("longitude must be a float")

        self._longitude = value
        self._to_cartesian_coord()

    @longitude.deleter
    def longitude(self):
        del self._longitude

    def _to_cartesian_coord(self):
        earth_radius = 6378
        lat_rad = self.latitude * math.pi/180
        longit_rad = self.longitude * math.pi/180
        self.cartesian_coord[0] = earth_radius * math.cos(lat_rad) * math.cos(longit_rad)
        self.cartesian_coord[1] = earth_radius * math.cos(lat_rad) * math.sin(longit_rad)

    def __init__(self, latitude: float, longitude: float):
        """OSM vertex, represents vertex using open street map data
        :param latitude: float, latitude of vertex
        :param longitude: float, longitude of vertex
        """
        self._latitude = 0
        self._longitude = 0
        self.cartesian_coord = [0, 0]
        self.latitude = latitude
        self.longitude = longitude
        self._to_cartesian_coord()


class OsmData:
    from typing import List
    VertexList = List[OsmVertex]
    EdgeList = List[OsmEdge]

    @property
    def vertices(self) -> VertexList:
        """List of OsmVertex objects
        :return List[OsmVertex]
        """
        return self._vertices

    @vertices.setter
    def vertices(self, vertices: VertexList):
        if not isinstance(vertices, list):
            raise ValueError("vertices must be a list of OsmVertex objects")

        lat_range = [math.inf, -math.inf]
        lon_range = [math.inf, -math.inf]
        cart_range_x = [math.inf, -math.inf]
        cart_range_y = [math.inf, -math.inf]
        # find the ranges of lat/lon and cartesian coordinates from new vertices
        for vertex in vertices:
            if not isinstance(vertex, OsmVertex):
                raise ValueError("vertices must be a list of OsmVertex objects")

            lat = vertex.latitude
            lon = vertex.longitude
            cart_x = vertex.cartesian_coord[0]
            cart_y = vertex.cartesian_coord[1]

            lat_range[0] = lat if lat_range[0] > lat else lat_range[0]
            lat_range[1] = lat if lat_range[1] < lat else lat_range[1]

            lon_range[0] = lon if lon_range[0] > lon else lon_range[0]
            lon_range[1] = lon if lon_range[1] < lon else lon_range[1]

            cart_range_x[0] = cart_x if cart_range_x[0] > cart_x else cart_range_x[0]
            cart_range_x[1] = cart_x if cart_range_x[1] < cart_x else cart_range_x[1]

            cart_range_y[0] = cart_y if cart_range_y[0] > cart_y else cart_range_y[0]
            cart_range_y[1] = cart_y if cart_range_y[1] < cart_y else cart_range_y[1]

        self.latitude_range = lat_range
        self.longitude_range = lon_range
        self.cartesian_range_x = cart_range_x
        self.cartesian_range_y = cart_range_y
        self._vertices = vertices

    @vertices.deleter
    def vertices(self):
        del self._vertices

    @property
    def edges(self) -> EdgeList:
        """List of OsmEdge objects
        :return: List[OsmEdge]
        """
        return self._edges

    @edges.setter
    def edges(self, edges: EdgeList):
        if not isinstance(edges, list):
            raise ValueError("edges must be a list of OsmVertex objects")
        for edge in edges:
            if not isinstance(edge, OsmEdge):
                raise ValueError("edges must be a list of OsmEdge objects")
        self._edges = edges

    @edges.deleter
    def edges(self):
        del self._edges

    def get_graph(self) -> GraphAdjList:
        """Construct a graph out of the vertex and edge
        data of the OSM object. The graph will associate the length
        of the edge to the graph edge. No data is bound to the vertices

        The vertices of the graph will be located at
        the location where given in the data set
        converted to cartesian coordinates
        :return: GraphAdjList
        """
        ret_graph = GraphAdjList()
        for k, vertex in enumerate(self.vertices):
            ret_graph.add_vertex(k, vertex)
            ret_graph.get_vertex(k).get_visualizer().set_location(vertex.cartesian_coord[0], vertex.cartesian_coord[1])
            ret_graph.get_vertex(k).get_visualizer().set_color("green")

        for k, edge in enumerate(self.edges):
            ret_graph.add_edge(edge.source, edge.destination, edge.distance)

        return ret_graph

    def __init__(self):
        self._vertices = []
        self._edges = []
        self.latitude_range = []
        self.longitude_range = []
        self.cartesian_range_x = []
        self.cartesian_range_y = []
        self.name = None



#!/usr/bin/env python
# package: bridges.base
from bridges.sl_element import *

##
# 	@brief This class can be used to instantiate Singly Linked
#	Circular List Elements.
#	Structurally they are the same as singly linked elements
#  except that each node constructed with the next point pointing to itself;
#  User's implementation of the circularly linked list needs to ensure that
#	the last node points to first node of the list, as the visualization generation is
#	dependent on this.
#
# 	Elements have labels (string) that are displayed on the visualization.
#  Elements take an generic object as a user defined parameter, E, which
#	can be any native type or object.
#
# 	Elements contains a visualizer (ElementVisualizer) object for setting visual
#	attributes (color, shape, opacity, size), necessary for displaying them in a web
#	browser.
#
#  Elements also have a LinkVisualizer object that is used when they are
#	linked to another element, appropriate for setting link attributes, between
#  an element and its next element.
#
#	@author Kalpathi Subramanian
#
#	@date 6/22/16, 1/7/17, 5/17/17
#
#
class CircSLelement(SLelement):
    ##
    #
    # 	This constructor creates an CircSLelement object
    # 	and sets its next pointer to itself
    # @param label the label of CircSLelement
    # @param e the generic object that this CircSLelement will hold
    # @param next the CircSLelement that should be assigned to the next pointer
    #
    def __init__(self, e = None, label = None, next = None):
        if e is not None and label is not None and next is None:
            super(CircSLelement, self).__init__(e = e, label = label)
            self.set_next(self)
        elif e is not None and label is None and next is not None:
            super(CircSLelement, self).__init__(e = e, next = next)
        elif e is None and label is None and next is not None:
            super(CircSLelement, self).__init__(next = next)
        else:
            super(CircSLelement, self).__init__()
            self.set_next(self)

    ##
    #
    #	This method gets the data structure type
    #
    #	@return  The date structure type as a string
    #
    #
    def get_data_structure_type(self):
        return "CircularSinglyLinkedList"

    ##
    #
    # Retrieves the next CircSLelement
    #
    # @return CircSLelement<E> assigned to next
    #
    #
    def get_next(self):
        return self.next

    #  (non-Javadoc)
    #
    # @see java.lang.Object#toString()
    #
    #
    def __str__(self):
        return "CircSLelement [next=" + self.next + ", getNext()=" + self.get_next() + ", getIdentifier()=" + self.get_identifier() + ", getVisualizer()=" + self.get_visualizer() + ", getClassName()=" + self.get_class_name() + ", getElementRepresentation()=" + self.get_element_representation() + ", getLabel()=" + self.get_label() + ", getValue()=" + self.get_value() + ", toString()=" + super(CircSLelement, self).__str__() + ", getClass()=" + self.get_class() + ", hashCode()=" + self.hash_code() + "]"



import json
from bridges.symbol import *

class Label(Symbol):

    def __init__(self, label = None):
        super(Label, self).__init__()
        if label is not None:
            self.set_label(label)
        self.width = 100
        self.height = 50
        self.font_size = 12

    def set_font_size(self, size):
        if size <= 0 or size > 200:
            raise ValueError("Please use font size between 0 and 200")
        else:
            self.font_size = size

    def get_dimensions(self):
        length = 0.09 * self.font_size * len(self.get_label())
        x = self.get_location()[0]
        y = self.get_location()[1]

        return [x - length/2, x + length/2, y, y]

    def get_json_representation(self):
        ds_json = super(Label,self).get_json_representation()
        ds_json["name"] = super(Label,self).get_label()
        ds_json["shape"] = "text"
        ds_json["font-size"] = self.font_size

        return ds_json



from bridges.symbol_collection import *
from bridges.bridges import *
from bridges.circle import *
from bridges.polygon import *
from bridges.rectangle import *
from bridges.label import *
from bridges.color import *

def main():
    bridges = Bridges(0, "test", "211416381091")
    bridges.connector.set_server("local")
    bridges.set_title("Symbol Collection")
    bridges.set_description("Some Shapes and Symbols")
    bridges.json_flag = True

    sc = SymbolCollection()
    s1 = Rectangle(50, 50, -25.0, 0.0)
    s1.set_fill_color(Color("green"))
    sc.add_symbol(s1)

    s2 = Circle(25.0, 0.0, 25)
    s2.set_fill_color(Color("green"))
    sc.add_symbol(s2)

    s3 = Polygon()
    s3.add_point(-100,0)
    s3.add_point(100, 0)
    s3.add_point(0, 0)
    s3.add_point(0, -100)
    s3.add_point(0, 100)
    s3.add_point(0, 0)
    s3.set_stroke_color(Color("magenta"))
    sc.add_symbol(s3)

    l = Label()
    l.set_location(0, 25)
    l.set_font_size(12)
    l.set_stroke_width(1.0)
    l.set_label("test label")
    l.set_stroke_color(Color("purple"))
    sc.add_symbol(l)

    bridges.set_data_structure(sc)
    bridges.visualize()

if __name__ == '__main__':
    main()


import json
from bridges.symbol import *

class Rectangle(Symbol):

    def __init__(self, w = None, h = None, locx = None, locy = None):
        super(Rectangle, self).__init__()
        if w is not None and h is not None:
            if w < 0 or h < 0:
                raise ValueError("Illegal height or width! Height and Width values need to be positive")
            self.width = w
            self.height = h
        if locx is not None and locy is not None:
            self.set_location(locx, locy)

    def get_name(self):
        return "rect"

    def set_width(self, w):
        if w <= 0 or w > 300:
            raise ValueError("Width need to be in the range(0-300)")
        self.width = w

    def set_height(self, h):
        if h <= 0 or h > 300:
            raise ValueError("Height need to be in the range(0-300)")
        self.height = h

    def get_dimensions(self):

        dims = []
        location = self.get_location()

        dims.append(location[0] - self.width/2)
        dims.append(location[0] - self.width/2)
        dims.append(location[1] - self.height/2)
        dims.append(location[1] - self.height/2)

        return dims

    def set_rectangle(self, locx, locy, w, h):
        self.set_location(locx, locy)
        if w <= 0 or w > 300 or h <= 0 or h > 300:
            raise ValueError("Height, Width need to be in the range(0-300)")
        self.width = w
        self.height = h

    def get_json_representation(self):

        ds_json = super(Rectangle, self).get_json_representation()
        ds_json["name"] = self.get_label()
        ds_json["shape"] = self.get_name()
        ds_json["width"] = self.width
        ds_json['height'] = self.height

        return ds_json






from bridges.bst_element import *
##
#  @brief This class extends the BSTElement class by adding a height and balance factor
#	fields that are useful in AVL trees.
#
#	AVL tree elements include a 'height' and a 'balFactor' value,
#	representing the height and balance factor of the AVL tree at
#  that node, respectively. This is useful in representing
#	AVL trees.
#
#	AVLTree elements contain a visualizer (ElementVisualizer) object for setting visual
#  attributes (color, shape, opacity, size), necessary for displaying them in a
#  web browser.
#
#  AVLTree elements also have a LinkVisualizer object, that is used when they are
#	linked to another element, appropriate for setting link attributes, for instance,
#	between *  the current element and its left or right child
#
#
#  @author Kalpathi Subramanian, Mihai Mehedint
#
#  @date 6/22/16, 1/7/17, 5/17/17
#
#
#
class AVLTreeElement(BSTElement):

    height = int()
    bal_factor = int()

    ##
    #
    # Construct an AVLTreeElement
    # @param k the search key
    # @param e the appl specific object that Element is holding
    #
    def __init__(self, k = None, e = None):
        if k is not None:
            super(AVLTreeElement, self).set_key(k)
        if e is not None:
            super(AVLTreeElement, self).__init__(key = k,e = e)
        else:
            super(AVLTreeElement, self).__init__()
        self.height = balFactor = 0


    ##
    #	This method gets the data structure type
    #
    #	@return  The date structure type as a string
    #
    #
    def get_data_structure_type(self):
        return "AVLTree"

    ##
    #	This method returns the height of the tree at this node
    #
    #	@return  height
    #
    #
    def get_height(self):
        return self.height

    ##
    #  This method sets the height of the tree at this node
    #
    #  @param   height h
    #
    #
    def set_height(self, h):
        self.height = h

    ##
    #	This method returns the balance factor  of the tree at this node
    #
    #   @return  balance factor
    #
    #
    def get_balance_factor(self):
        return self.bal_factor

    ##
    #	This method sets the balance factor of the tree at this node
    #
    #  @param   balance factor  bf
    #
    def set_balance_factor(self, bf):
        self.bal_factor = bf

    ##
    #
    # This method returns the left child of the tree node
    #
    # @return the left child of this node
    #
    #
    def get_left(self):
        return super(AVLTreeElement, self).get_left()

    ##
    #
    #  This method returns the right child of tree node
    #
    # @return the right child of this node
    #
    #
    def get_right(self):
        return super(AVLTreeElement, self).get_right()



from bridges.color import Color

my_color = Color()

my_color.red = 10.5
my_color.green = 100
my_color.blue = 255
my_color.alpha = 1.0

print(my_color.rgba)

my_color.set_color(r=0, g=24.5, b=32, a=0)
print(my_color.rgba)

my_color = Color(red=0, green=0, blue=0, alpha=1.0)
print(my_color.rgba)

named_color = Color(col_name="blue")
print(named_color.rgba)

my_color = Color(b=255)
print(my_color.rgba)

my_color = Color(blue=255)
print(my_color.rgba)

named_color = Color("red")
print(named_color.rgba)

try:
    named_color = Color("Not a Real Color")
except ValueError:
    print("Passed invalid color name test")

other_color = Color()
other_color.rgba = 100, 0, 20, 0
print(my_color.rgba, other_color.rgba)

other_color.set_red(255)
other_color.set_green(255)
other_color.set_blue(255)
other_color.set_alpha(1.0)
print(other_color.rgba)

other_color.set_color(0, 0, 0, 0)
print(other_color.rgba)

try:
    my_color.rgba = 256, 0, 1, 1.0
except ValueError:
    try:
        my_color.rgba = 0, 256, 0, 1.0
    except ValueError:
        try:
            my_color.rgba = 0, 0, 256, 1.0
        except ValueError:
            try:
                my_color.rgba = 0, 0, 0, 1.1
            except ValueError:
                print("Setter passed for overflow on RGBA")
else:
    print("Setter failed for overflow on RGBA")

try:
    my_color.rgba = -1, 0, 255, 0.0
except ValueError:
    try:
        my_color.rgba = 0, -1, 0, 1.0
    except ValueError:
        try:
            my_color.rgba = 0, 0, -1, 1.0
        except ValueError:
            try:
                my_color.rgba = 0, 0, 0, -0.1
            except ValueError:
                print("Setter passed for underflow on RGBA")
else:
    print("Setter failed for underflow on RGBA")



#!/usr/bin/env python
from bridges.element_visualizer import *
from bridges.color import *
from bridges.link_visualizer import *
import re
from decimal import Decimal
import traceback
import sys
# from bridges.kd_tree_element import *



##
# @brief This is the main superclass in BRIDGES for  deriving a number of
# 	objects used  in building arrays, lists, trees and graph data structures.
#  SLelement, DLelement, CircSLelement, CircDLelement, TreeElement, BinTreeElement,
#	BSTElement, CircSLelement, CircDLelement, AVLTreeElement are all subclasses
#  (see class hierarchy above).  Element contains  two
#	visualizer objects (ElementVisualizer, LinkVisualizer) for specifying
#	visual attributes for nodes and links respectively. It also contains a label that
#	that can be displayed in BRIDGES visualizations.
#
#  All the tutorials under
#
#	http://bridgesuncc.github.io/Hello_World_Tutorials/Overview.html
#
#  illustrate examples of using different types of Element objects and how to
#	manipulate their visual attributes.
#
# @author Mihai Mehedint, Kalpathi Subramanian
#
#
class Element(object):

    QUOTE = "\""
    COMMA = ","
    COLON = ":"
    OPEN_CURLY = "{"
    CLOSE_CURLY = "}"
    OPEN_PAREN = "("
    CLOSE_PAREN = ")"
    OPEN_BOX = "["
    CLOSE_BOX = "]"

    ids = 0
    label = str()
    identifier = str()
    # visualizer = ElementVisualizer()
    # print(visualizer)
    # lvisualizer = dict()
    value = object()

    #
    # 	this is the number of pattern matches where the new string
    #  	can be inserted; useful in case we insert line breaks at a
    #  	desired number of characters is the pattern is change to
    # 	white space this index can be changed to 2 words to insert a
    # 	line break every 2 words
    word_number = 0

    # this is the string value that replaces the pattern found in the label
    INSERT_STRING = "\\n"

    # 	for more complex patterns the key must be changed
    # 	like so "((John) (.+?))" returns "John firstWordAfterJohn":
    # 	John writes, John doe, John eats etc.
    # 	(\\w) matches any word (\\d) any digit (\\D) any non digit
    # 	(\\s) a white space (\\s*) zero or more whitespaces, (\\s+)
    # 	one or more
    DIVIDE_KEY = "(\r?\n)|(\n)|(\f)|(\r)|(%n)"

    # public String INSERT_STRING = "\\n";
    # public String DIVIDE_KEY ="(\n)";
    def get_data_structure_type(self):
        return "Element"

    ##
    # Element constructor
    # creates an ElementVisualizer object
    # sets a unique identifier for the current Element
    # normally used from subclasses
    # @param val generic parameter value used to construct Element
    # @param label the string that is visible on the bridges Visualization
    #
    def __init__(self, label = None, val=None, original=None):
        Element.ids += 1
        self.visualizer = ElementVisualizer()
        self.lvisualizer = dict()

        if val is not None and label is None and original is None:
            self.set_value(val)
            self.identifier = str(Element.ids)
            self.label = "Default"
            self.set_visualizer(ElementVisualizer())
            self.ids = Element.ids
            self.lvisualizer = dict()
        if val is not None and label is not None:
            self.identifier = str(Element.ids)
            self.label = "Default"
            self.set_visualizer(ElementVisualizer())
            self.ids = Element.ids
            self.lvisualizer = dict()
            self.set_label(label)
            self.set_value(val)
        if original is not None:
            self.identifier = str(Element.ids)
            self.ids = Element.ids
            self.label = str(original.get_label())
            self.visualizer = ElementVisualizer(original.get_visualizer())
            self.lvisualizer = dict()
            self.set_value(original.get_value())
        if val is None and label is None and original is None:
            self.identifier = str(Element.ids)
            self.label = "Default"
            self.set_visualizer(ElementVisualizer())
            self.ids = Element.ids
            self.lvisualizer = dict()


    ##
    # this method returns the element's unique identifier
    # @return the string identifier
    #
    def get_identifier(self):
        return self.identifier

    ##
    # Returns the Element's visualizer object
    #
    # @return the visualizer object
    #
    def get_visualizer(self) -> ElementVisualizer:
        return self.visualizer

    ##
    # This method sets the visualizer object for the current
    # element object
    #
    # @param visualizer the visualizer to set
    #
    def set_visualizer(self, visualizer):
        self.visualizer = visualizer

    ##
    # Returns the Element's link visualizer object
    #
    # The link visualizer object links this element to another element, which
    # is specified by the argument to this method. This method is typically used
    # to set the visual attributes of the links, such as in graphs or binary tree
    # structures.
    #
    # @parm Element el -- the element terminating the link
    #
    # @return the link visualizer
    #
    def get_link_visualizer(self, el):
        #  if this is the first time, must create the
        #  link visualizer
        try:
            if (type(el) != Element):
                raise ValueError("Wrong type, Needs to be Element.")
        except Exception as e:
            tb = sys.exc_info()
            traceback.print_tb(tb)
        if el in self.lvisualizer:
            return self.lvisualizer[el]
        else:
            self.lvisualizer[el] = LinkVisualizer()
            return self.lvisualizer[el]

    ##
    #	Sets the link from this element to a new incoming element
    #
    #	@param el the element to be linked to.
    #
    #
    def set_link_visualizer(self, el):
        self.lvisualizer[el] = LinkVisualizer()

    def remove_link_visualizer(self, el):
        self.lvisualizer.pop(el)

    ##
    # Validates the Element's value when the Element is created
    # A non null value is expected
    # this will be unnecessary after we modify the server
    # @param Element value
    #
    # def validate_val(self, value):
    #     try:
    #         if value == None:
    #             raise NullPointerException("\nInvalid value set to Element<E> '" + value + "'. Expected" + " non null E value.\n")
    #         elif value.__class__.getCanonicalName().isEmpty():
    #             raise IllegalArgumentException("\nThe argument is not a legal Element object!\n" + value.__class__.getCanonicalName())
    #         else:
    #     except Exception as e:
    #         e.print_stack_trace()
    def get_class_name(self):
        return self.value.__class__.__name__


    def get_element_representation(self):
        json_str = self.OPEN_CURLY + self.QUOTE + "name" + self.QUOTE + self.COLON + self.QUOTE + str(self.label) + self.QUOTE + self.COMMA + self.QUOTE + "shape" + self.QUOTE + self.COLON + self.QUOTE + self.visualizer.get_shape() + self.QUOTE + self.COMMA + self.QUOTE + "size" + self.QUOTE + self.COLON + str(self.visualizer.get_size()) + self.COMMA + self.QUOTE + "color" + self.QUOTE + self.COLON + self.OPEN_BOX + str(self.visualizer.get_color().get_red()) + self.COMMA + str(self.visualizer.get_color().get_green()) + self.COMMA + str(self.visualizer.get_color().get_blue()) + self.COMMA + str(self.visualizer.get_color().get_alpha()) + self.CLOSE_BOX
        loc_flag = not ((self.visualizer.get_locationX() == Decimal('Infinity')) or (self.visualizer.get_locationY() == Decimal('Infinity')))
        if loc_flag:
             json_str += self.COMMA + self.QUOTE + "location" + self.QUOTE + self.COLON + self.OPEN_BOX + str(self.visualizer.get_locationX()) + self.COMMA + str(self.visualizer.get_locationY()) + self.CLOSE_BOX
        if self.get_data_structure_type() == "KDTree":
            kdt = self
            json_str += self.COMMA + self.QUOTE + "key" + self.QUOTE + self.COLON + self.QUOTE + str(kdt.get_key()) + self.QUOTE + self.COMMA + self.QUOTE + "dimension" + self.QUOTE + self.COLON + self.QUOTE + str(kdt.get_dimension()) + self.QUOTE + self.COMMA + self.QUOTE + "thickness" + self.QUOTE + self.COLON + self.QUOTE + str(kdt.get_thickness()) + self.QUOTE
        if self.get_data_structure_type() == "BinarySearchTree":
            bst = self
            json_str += self.COMMA + self.QUOTE + "key" + self.QUOTE + self.COLON + self.QUOTE + str(bst.get_key()) + self.QUOTE + self.COMMA
        elif self.get_data_structure_type() == "AVLTree":
            avl = self
            json_str += self.COMMA + self.QUOTE + "key" + self.QUOTE + self.COLON + self.QUOTE + str(avl.get_key()) + self.QUOTE + self.COMMA + self.QUOTE + "height" + self.QUOTE + self.COLON + str(avl.get_height()) + self.COMMA + self.QUOTE + "balance_factor" + self.QUOTE + self.COLON + str(avl.get_balance_factor()) + self.COMMA
        else:
            json_str += self.CLOSE_CURLY
        return json_str

    def get_link_representation(self, lv, src, dest):
        return self.OPEN_CURLY + lv.get_link_properties() + self.COMMA + self.QUOTE + "source" + self.QUOTE + self.COLON + src + self.COMMA + self.QUOTE + "target" + self.QUOTE + self.COLON + dest + self.CLOSE_CURLY

    ##
	#  This method returns the existing value of the label fields
	#
	#  @return the label of the Element; the label is typically displayed on BRIDGES
	#  			visualizations.
	#
    def get_label(self):
        return self.label

    ##
	#  This method sets the label
	#
	#  @param label the label to set
	#
    def set_label(self, label):
        self.label = self.arrange_label(label, self.word_number)

    ##
	#  This method formats the label string using a predefine pattern (DIVIDE_KEY) and
	#  replaces the pattern with the string characters hold by the INSERT_STRING global
	#  variable
	#
	#  @param label  the input label string
	#
	#  @param wordNumber in very long strings in the case where the whitespace
	#  \\s is chosen as a key the wordNumber can be set
	#  to replace the whitespace with a newline character \\n at a given number of
	#  words (every second or third word)
	#  The default value is 0. In most situations we want to replace all patterns found.
	#  for more complex patterns the key must be changed like so "((John) (.+?))"
	#  returns "John firstWordAfterJohn": John writes, John doe, John eats etc.
	#  (\\w) matches any word (\\s) one white space (\\s*) zero or more white spaces,
	#  (\\s+) one or more
	#
	#  @return  the formatted label
	#
    def arrange_label(self, label, word_number):
        my_pattern = re.compile(self.DIVIDE_KEY)
        # match = my_pattern.findall(str(label))
        # if not len(match) == 0:
        return label
        # else:
        #     print ("error")
        #     return str()
            # match.reset()
            # while match.find():
            #     counter += 1
            #     if counter == wordNumber:
            #         counter = -1
            #         match.appendReplacement(str, Matcher.quoteReplacement(self.INSERT_STRING))
            # match.appendTail(str)
            # if len(str) == 0:
            #     return label
            # else:
            #     return label = str.__str__()

    ##
	#  This method returns the generic parameter value held in the element.
	#
	#  @return the value
	#
    def get_value(self):
        return self.value

    ##
	#  This method sets the generic parameter value for  this element.
	#
	#  @param value the value to set
	#
    def set_value(self, value):
        # self.validateVal(value)
        self.value = value

    def __str__(self):
        return "Element [name=" + self.label + ", identifier=" + self.identifier + ", visualizer=" + str(self.visualizer) + ", value=" + self.value + ", getIdentifier()=" + self.get_identifier() + ", getVisualizer()=" + str(self.get_visualizer()) + ", getClassName()=" + self.get_class_name() + ", getElementRepresentation()=" + self.get_element_representation() + ", getLabel()=" + self.get_label() + ", getValue()=" + self.get_value() + ", getClass()=" + self.get_class_name() + ", hashCode()=" + self.hash_code() + ", toString()=" + super(Element, self).__str__() + "]"




##
#  @brief A class to hold actor movie data -- using IMDB dataset
#
#
#  @author Kalpathi Subramanian
#  @date 2/18/18
#
#

class ActorMovieIMDB():

    ##
	# This method initializes an actor movie object with an actor/movie pair.
	#
	# @param a - actor  (string)
	# @param m - movie  (string)
	# @param r - rating  (float)
    # @param genr - genres  (vector)
    def __init__(self, a = None, m = None, r = None, genr = None):
        if a is not None:
            self.actor = a
        if m is not None:
            self.movie = m
        if r is not None:
            self.rating = r
        if genr is not None:
            self.genres = genr

    ##
	#
	#  	 Get actor name
	#
	#  	 @return actor name (string)
	#
	#
    def get_actor(self):
        return self.actor

    ##
	#
	#  	 Set actor name
	#
	#  	 @param actor name (string)
	#
	#
    def set_actor(self, a):
        self.actor = a

    ##
	#
	#  	 Get movie name
	#
	#  	 @return movie name (string)
	#
	#
    def get_movie(self):
        return self.movie

    ##
	#
	#  	 Set movie name
	#
	#  	 @param movie name (string)
	#
	#
    def set_movie(self, m):
        self.movie = m

    ##
	#
	#  	 Get movie rating
	#
	#  	 @return movie rating  (double)
	#
	#
    def get_rating(self):
        return self.rating

    ##
	#
	#  	 Set movie rating
	#
	#  	 @param movie rating (double)
	#
	#
    def set_rating(self, r):
        self.rating = r

    ##
	#
	#  	 Get movie genres
	#
	#  	 @return movie genres  (vector)
	#
	#
    def get_genres(self):
        return self.genres

    ##
	#
	#  	 Set movie genres
	#
	#  	 @param movie genres (vector)
	#
	#  	
    def set_genres(self, genr):
        self.genres = genr



#!/usr/bin/env python
from bridges.sl_element import *

##
# 	@brief This class can be used to instantiate Multi-list Elements.
# 	This class extends SLelement (singly linked list element) to build multi-lists;
#	Multilist elements contain a tag that indicates if the element is a sublist or not;
#	If the element points to a sublist, then the sublist field is the beginning of
#	this sublist. If not, the data field contains the user specified data item and
#	list continues (getNext()/setNext()). As in singly linked elements, the next pointer
#	points to the following list element of the list or sublist.
#
# 	Multi-list elements contain a visualizer (ElementVisualizer) object for setting
#	visual attributes (color, shape, opacity, size), necessary for displaying
#	them in a web browser.
#
#	Elements also have a LinkVisualizer object, that is used when they are linked to
#	another element, appropriate for setting link attributes, for instance, between
#	the current element and its next element. In this case, the link in question is that
#  which connects the element to the following elements; a similar logic follows for
#	sublists.
#
# @author , Kalpathi Subramanian
#
#
#	\sa Example Tutorial at <br> ??
#
class MLelement(SLelement):
    sub_list = None

    #  link to a sublist
    tag = False

    ##
    #
    # This constructor creates an SLelement object
    # and sets the next pointer to null
    # @param label the label of SLelement that shows up on the bridges visualization
    # @param e the generic object that this SLelement will hold
    # @param next the element that should be assigned to the next pointer
    # @param sublist the MLelement that is the beginning of a sublist
    #
    def __init__(self, label = None, e = None, next = None, sublist = None):
        if label and e is not None:
            super(MLelement, self).__init__(e = e, label = label)
            self.sub_list = None
        if e is not None and label is None:
            super(MLelement, self).__init__(e = e, label = "")
        if next and sublist is not None:
            super(MLelement, self).__init__()
            self.set_next(next)
            self.sub_list = sublist
            if sublist is not None:
                self.tag = True
                self.set_link_visualizer(sublist)
        if label is None and e is None and next is None and sublist is None:
            super(MLelement, self).__init__()
            self.sub_list = None

    ##
    # Sets the start of a new sublist.
    # to the SLelement "next"
    #
    # @param sl the MLelement that is the beginning of a sublist
    #
    def set_sub_list(self, sl):
        self.sub_list = sl
        if sl is not None:
            self.tag = True
            self.set_link_visualizer(sl)
        #  by default, color and shape sublist nodes to distinguish them  from
        # 	remaining nodes
        self.get_visualizer().set_color("red")
        self.get_visualizer().set_shape("square")

    ##
	#  Gets the sublist at this node, if it exists
	#
	#  @return  the sublist head element, if it exists
	#
    def get_sub_list(self):
        return self.sub_list

    ##
	#  This method gets the data structure type
	#
	#  @return  The date structure type as a string
	#
	#
    def get_data_structure_type(self):
        return "MultiList"

    ##
	#  Retrieves the element following this element
	#
	#  @return MLelement<E> assigned to next
	#
	#
    def get_next(self):
        return self.next

    ##
	#
	#  Sets the tag of the element.
	#
	#  @param boolean t
	#
    def set_tag(self, t):
        self.tag = t

    ##
	#  Gets the tag of the element.
	#
	#  @return tag of the element
	#
    def get_tag(self):
        return self.tag

    def get_data_structure_representation(self):
        nodes = []
        nodes.clear()
        self.get_list_elements(nodes)
        nodes_JSON = str()
        node_map = dict()
        k = 0
        while k < len(nodes):
            node_map[nodes[k]] = k
            nodes_JSON+=(nodes[k].get_element_representation())
            nodes_JSON+=(self.COMMA)
            k += 1
        if len(nodes) != 0:
            nodes_JSON = nodes_JSON[:-1]
        links_JSON = str()
        k = 0
        while k < len(nodes):
            par = nodes[k]
            if par.tag:
                chld = par.sub_list
                #  sub list
                if chld is not None:
                    #  add the link
                    links_JSON+=(self.get_link_representation(par.get_link_visualizer(chld), str(node_map.get(par)), str(node_map.get(chld))))
                    links_JSON+=(self.COMMA)
            chld = par.next
            if chld is not None:
                #  add the link
                links_JSON+=(self.get_link_representation(par.get_link_visualizer(chld), str(node_map.get(par)), str(node_map.get(chld))))
                links_JSON+=(self.COMMA)
            k += 1
        if len(links_JSON) > 0:
            links_JSON = links_JSON[:-1]
        json_str = self.QUOTE + "nodes" + self.QUOTE + self.COLON + self.OPEN_BOX + nodes_JSON.__str__() + self.CLOSE_BOX + self.COMMA + self.QUOTE + "links" + self.QUOTE + self.COLON + self.OPEN_BOX + links_JSON.__str__() + self.CLOSE_BOX + self.CLOSE_CURLY
        return json_str

    ##
	#  Get the elements of the list
	#
	#  @param nodes  a vector of the ndoes in the list
	#
    def get_list_elements(self, nodes):
        self.get_list_elements_R(self, nodes)

    
    def get_list_elements_R(self, list, nodes):
        el = list
        while el is not None:
            nodes.append(el)
            if el.tag:
                self.get_list_elements_R(el.sub_list, nodes)
            el = el.get_next()



#!/usr/bin/env python
# package: bridges.base
from bridges.sl_element import *


##
#
# 	@author Mihai Mehedint, Kalpathi Subramanian
#
#
# 	@brief This class is used to create doubly linked element objects.
#
#	This class extends Element and takes a generic parameter <E> representing
#	application specific data. This element forms the basic building block for
#	doubly linked lists. Doubly linked elements have two links,
# 	"next" and "previous", that point to the previous and succeeding nodes along the list.
#
#	Elements contain a visualizer (ElementVisualizer) object for setting visual
#	attributes (color, shape, opacity, size), necessary for displaying them in a web
#	browser.
#
#	Elements also have a LinkVisualizer object that is used when they are linked to
#	another element, appropriate for setting link attributes, such as in linked lists,
#	between the current element and its next or previous nodes.
#
#
class DLelement(SLelement):
    prev = object()

    ##
    #	Constructs an empty DLelement.
    # @param e the genereic object that this DLelement is holding
	# @param next the DLelement that should be assigned to the next pointer
	# @param prev the DLelement that should be assigned to the prev pointer
    # @param label the label for this DLelement
    #
    def __init__(self, e = None, label = None, next = None, prev = None):
        if next is not None:
            self.set_next(next)
        if prev is not None:
            self.set_prev(prev)
        if e is not None and label is not None:
            super(DLelement, self).__init__(e = e, label = label)
            self.prev = None
        elif e is not None and label is None:
            super(DLelement, self).__init__(e = e)
        else:
            self.prev = None
            super(DLelement, self).__init__()

    ##
    #	This method gets the data structure type
    #
    #	@return  The date structure type as a string
    #
    def get_data_structure_type(self):
        return "DoublyLinkedList"

    ##
    # This method returns the pointer to the next DLelement
    #
    # @return the DLelement assigned to the next pointer
    #
    def get_next(self):
        return self.next

    ##
    # This method sets the pointer to the next DLelement
    #
    # @param next the DLelement that should be assigned to the next pointer
    #
    #
    #
    # 		public void setNext(DLelement<E> nxt) {
    # 			this.next = nxt;
    # 			if (nxt != null)
    # 				this.setLinkVisualizer(nxt);
    # 		}
    #
    #
    # This method returns the pointer to the previous DLelement
    #
    # @return the DLelement assigned to the prev pointer
    #
    def get_prev(self):
        return self.prev

    ##
    # This method sets the pointer to the previous DLelement
    #
    # @param prev the DLelement that should be assigned to the prev pointer
    #
    def set_prev(self, prv):
        #  first remove any existing link visualizer from this node
        # 	to its next node
        if self.prev is not None:
            self.remove_link_visualizer(self.prev)
        self.prev = prv
        #  add a new link visualizer
        if prv is not None:
            self.set_link_visualizer(prv)

    ##
    #	Get the JSON representation of the the data structure
    #
    def get_data_structure_representation(self):
        #  map to reorder the nodes for building JSON
        node_map = dict()
        #  get teh list nodes
        nodes = []
        SLelement.get_list_elements(self, nodes)
        #  generate the JSON of the list nodes
        nodes_JSON = str()
        k = 0
        while k < len(nodes):
            node_map[nodes[k]] = k
            nodes_JSON += nodes[k].get_element_representation()
            nodes_JSON += self.COMMA
            k += 1
        #  remove the last comma
        if len(nodes) != 0:
            nodes_JSON = nodes_JSON[:-1]
        links_JSON = str()
        k = 0
        while k < len(nodes):
            par = nodes[k]
            nxt = par.next
            prv = par.prev
            if nxt is not None:
                #  add the link
                links_JSON += self.get_link_representation(par.get_link_visualizer(nxt),str(node_map.get(par)), str(node_map.get(nxt)))
                links_JSON += self.COMMA
            if prv is not None:
                #  add the link
                links_JSON += (self.get_link_representation(par.get_link_visualizer(prv), str(node_map.get(par)), str(node_map.get(prv))))
                links_JSON += (self.COMMA)
            k += 1
        #  add the link
        #  add the link
        if len(links_JSON) > 0:
            links_JSON = links_JSON[:-1]
        json_str = self.QUOTE + "nodes" + self.QUOTE + self.COLON + self.OPEN_BOX + str(nodes_JSON) + self.CLOSE_BOX + self.COMMA + self.QUOTE + "links" + self.QUOTE + self.COLON + self.OPEN_BOX + str(links_JSON) + self.CLOSE_BOX + self.CLOSE_CURLY
        return json_str



#!/usr/bin/env python
import webcolors


class Color(object):
    """This class is used to represent colors in bridges.

    We use and RGBA model to represent colors, with the Red Green and Blue components ranging from 0-255,
    with the alpha ranging from 0.0-1.0 inclusive.

    We use webcolors to handle color names passed to the constructor/set_color function.
    https://webcolors.readthedocs.io/en/1.8.1/
    All CSS3 color names should be valid:
    https://developer.mozilla.org/en-US/docs/Web/CSS/color_value
    https://www.w3.org/TR/css-color-3/#svg-color

    Attributes:
        red (int): red component of color ranging from 0-255 inclusive (default 0)
        green (int): green component of color ranging from 0-255 inclusive (default 0)
        blue (int): blue component of color ranging from 0-255 inclusive (default 0)
        alpha (float): alpha component of color ranging from 0.0-1.0 inclusive (default 1.0)
        rgba (tuple(int, int, int, alpha)): RGBA components as respective tuple
    Args:
        args: int, int, int, Optional(float) or a str as singular arg
        kwargs:
            r or red: Optional(int)
            b or blue: Optional(int)
            g or green: Optional(int)
            a or alpha: Optional(float)
            col_name: Optional(str)
    Raises:
        ValueError: if args is not 3 ints with an optional 4th arg for alpha or just one str arg
        ValueError: if a str passed is not a valid webcolor
        ValueError: if any of the RGBA values are outside of their respective range
    Examples:
        >>> my_color = Color("red")
        >>> my_color.rgba
        (255, 0, 0, 1.0)
        >>> my_color = Color(r=255)
        >>> my_color.rgba
        (255, 0, 0, 1.0)
        >>> my_color = Color(255, 0, 0)
        >>> my_color.rgba
        (255, 0, 0, 1.0)
        >>> my_color = Color()
        >>> my_color.red = 255
        >>> my_color.rgba
        (255, 0, 0, 1.0)
    """
    @property
    def red(self) -> int:
        """red component of color
        :return int: red component of color
        Must be a value between 0-255 inclusive
        """
        return self._red

    @red.setter
    def red(self, value: int):
        try:
            value = int(value)
        except ValueError:
            raise ValueError("Value for RGB attributes must be able to be casted to int")
        if value < 0 or value > 255:
            raise ValueError("Value for RGB attributes should range from 0-255 inclusive")

        self._red = value

    @red.deleter
    def red(self):
        del self._red

    @property
    def green(self) -> int:
        """green component of color
        :return int: green component of color
        Must be a value between 0-255 inclusive
        """
        return self._green

    @green.setter
    def green(self, value: int):
        try:
            value = int(value)
        except ValueError:
            raise ValueError("Value for RGB attributes must able be to be casted to int")
        if value < 0 or value > 255:
            raise ValueError("Value for RGB attributes should range from 0-255 inclusive")

        self._green = int(value)

    @green.deleter
    def green(self):
        del self._green

    @property
    def blue(self) -> int:
        """blue component of color
        :return int: blue component of color
        Must be a value between 0-255 inclusive
        """
        return self._blue

    @blue.setter
    def blue(self, value: int):
        try:
            value = int(value)
        except ValueError:
            raise ValueError("Value for RGB attributes must able be to be casted to int")
        if value < 0 or value > 255:
            raise ValueError("Value for RGB attributes should range from 0-255 inclusive")

        self._blue = int(value)

    @blue.deleter
    def blue(self):
        del self._blue

    @property
    def alpha(self) -> float:
        """alpha component of color
        :return float: alpha component of color
        Must be a value between 0.0-1.0 inclusive
        """
        return self._alpha

    @alpha.setter
    def alpha(self, value: float):
        try:
            value = float(value)
        except ValueError:
            raise ValueError("Value for Alpha must be able to be casted to float")
        if value < 0.0 or value > 1.0:
            raise ValueError("Value for alpha should range from 0.0 - 1.0 inclusive")

        self._alpha = value

    @alpha.deleter
    def alpha(self):
        del self._alpha

    @property
    def rgba(self) -> (int, int, int, float):
        """RGBA components as respective tuple
        Represents the RGBA values of the color as a tuple, can be used to set or get all values at once
        :return (int, int, int, float): RGBA values respectively
        """
        return self.red, self.green, self.blue, self.alpha

    @rgba.setter
    def rgba(self, rgba: (int, int, int, float)):
            self.red, self.blue, self.green, self.alpha = rgba

    @rgba.deleter
    def rgba(self):
        del self.red
        del self.blue
        del self.green
        del self.alpha

    def __init__(self, *args, **kwargs):
        """ Constructor for a Color object
        Usage: requires either 3 ints 0-255 for RGB and an optional float 0.0-1.0 for alpha or a str of a web color
        can also key the RGBA values with r, g, b, a or red, green, blue, alpha respectively and col_name for the str
        :param args: int, int, int, optional float or just a str
        :param kwargs: r/red: int, b/blue: int, g/green: int optional a/alpha: float or col_name: str
        :return: None
        """
        self._red = 0
        self._green = 0
        self._blue = 0
        self._alpha = 1.0
        self.set_color(*args, **kwargs)

    def set_color(self, *args, **kwargs) -> None:
        """
        Usage: requires either 3 ints 0-255 for RGB and an optional float 0.0-1.0 for alpha or a str of a web color
        can also key the RGBA values with r, g, b, a or red, green, blue, alpha respectively and col_name for the str
        :param args: int, int, int optional float or str
        :param kwargs: r/red: int, b/blue: int, g/green: int optional a/alpha: float or col_name: str
        :return: None
        """
        col_name = None
        if args:
            if len(args) == 4 or len(args) == 3:
                self.red = args[0]
                self.green = args[1]
                self.blue = args[2]
                if len(args) == 4:
                    self.alpha = args[3]
            elif len(args) == 1:
                if type(args[0]) is str:
                    col_name = args[0]
            else:
                raise ValueError("To use Color constructor pass 3 RGB values and a float alpha value or a color name")
        elif kwargs:
            if 'col_name' in kwargs:
                col_name = kwargs['col_name']
            if 'r' in kwargs:
                self.red = kwargs['r']
            if 'red' in kwargs:
                self.red = kwargs['red']
            if 'g' in kwargs:
                self.green = kwargs['g']
            if 'green' in kwargs:
                self.green = kwargs['green']
            if 'b' in kwargs:
                self.blue = kwargs['b']
            if 'blue' in kwargs:
                self.blue = kwargs['blue']
            if 'a' in kwargs:
                self.alpha = kwargs['a']
            if 'alpha' in kwargs:
                self.alpha = kwargs['alpha']

        if col_name is not None:
            try:
                web_color = webcolors.name_to_rgb(col_name)
                self.set_color(web_color.red, web_color.green, web_color.blue)
            except ValueError:
                raise ValueError(col_name + " is not a valid color name")

    def set_red(self, r: int) -> None:
        """:param int r: Must be a value between 0-255 inclusive"""
        self.red = r

    def get_red(self) -> int:
        """:return int: red component of color"""
        return self.red

    def set_green(self, g: int) -> None:
        """:param int g: Must be a value between 0-255 inclusive"""
        self.green = g

    def get_green(self) -> int:
        """:return int: green component of color"""
        return self.green

    def set_blue(self, b: int) -> None:
        """:param int b: Must be a value between 0-255 inclusive"""
        self.blue = b

    def get_blue(self) -> int:
        """":return int: blue component of color"""
        return self.blue

    def set_alpha(self, a: float) -> None:
        """:param float a: Must be a value between 0.0-1.0 inclusive"""
        self.alpha = a

    def get_alpha(self) -> float:
        """:return float: alpha component of color"""
        return self.alpha

    def get_byte_representation(self) -> list:
        """:return list(int):RGBA values as list of ints from 0-255"""
        r = self.red
        g = self.green
        b = self.blue
        a = round(255 * self.alpha)

        bytebuffer = list()
        bytebuffer.append(r)
        bytebuffer.append(g)
        bytebuffer.append(b)
        bytebuffer.append(a)

        return bytebuffer

    def __eq__(self, other):
        """deep equality check, by value of each RGBA value"""
        if not isinstance(other, self.__class__):
            return False

        return (self.red == other.red and
                self.green == other.green and
                self.blue == other.blue and
                self.alpha == other.alpha)



#!/usr/bin/env python
## package: bridges.base
from bridges.color import *

# from Element import Element

##
#  @brief This class maintains the visual attributes of links that join
#  bridges elements.
#
#  Visual properties include color, thickness, and opacity.
#  Objects of this class are stored as part of the Element class.
#  Generally, a user will manipulate the LinkVisualizer returned from the
#  Element's getLinkVisualizer(Element it) method (which it is the bridges element
#	this element is linked to), and then set attributes using its methods. Links are
#  utilized in all types of linked lists, tree and graph structures.
#
#  Supported attribute values are as follows:<p>
#
#  <b>Supported Colors (by name)</b>: <p>
#  "red", "green", "blue","yellow","cyan","magenta",
#  "white",, "black", "orange",  "turquoise",  "maroon",  <br>
#  "aquamarine",  "azure",  "beige", "brown",  "tan",  "olive",
#  "chartreuse", "khaki", "bisque",  "coral", <br>
#  "pink",  "lavender",  "purple",  "gold" <p>
#
#  <b> Color by RGBA Specification :</b>  Range: 0-255 for each component <p>
#
#  <b> Thickness: </b> Range : 0.0-50.0
#
#  <b> Opacity: </b> Range (0.0-1.0) </p>
#
#  @author Mihai Mehedint, Kalpathi Subramanian
#
#
#  \sa Example Tutorial at <br>
#  http://bridgesuncc.github.io/Hello_World_Tutorials/SLL.html
#
#
class LinkVisualizer(object):
    #  Link visualization properties for this element.
    #  maintains mapping from the terminating vertex to its
    #  visual properties
    #  implemented as a hashmap mapping into properties, which
    #  is als a hashmap, to keep the accesses constant time.
    QUOTE = "\""
    COMMA = ","
    COLON = ":"
    OPEN_CURLY = "{"
    CLOSE_CURLY = "}"
    OPEN_PAREN = "("
    CLOSE_PAREN = ")"
    OPEN_BOX = "["
    CLOSE_BOX = "]"

    #  link color
    # color = Color.Color(None, 70, 130, 180, 1.0)

    #  link thickness
    thickness = float()

    #  link weight
    weight = float()

    def __init__(self):
        # super(LinkVisualizer.LinkVisualizer, self).__init__()
        self.color = Color(70, 130, 180, 1.0)
        self.set_color(70, 130, 180, 1.0)
        self.set_thickness(1.0)
        self.set_weight(1.0)

    ##
    # Set the thickness of the link in the Bridge Visualization in pixels; thickness
    # shoudl be in the range 0-50.0
    #
    # @param thickness
    #
    #
    def set_thickness(self, th):
        # Validation.validateThickness(th)
        self.thickness = th

    ##
    # Get the thickness of the link in the bridges Visualiation
    #
    # @return the size in pixels of the Element in the bridges Visualization
    #
    def get_thickness(self):
        return self.thickness

    ##
    # Set the weight of the link, useful in graph algorithms, for example.
    # weight value is user defined, and determined by the input graph specification.
    #
    # @param weight
    #
    #
    def set_weight(self, wt):
        #  is user defined so no checking
        self.weight = wt

    ##
    # Get the weight of the link
    #
    # @return the stored edge weight
    #
    def get_weight(self):
        return self.weight

    ##
    #
    #	Set the color of the link in the bridges Visualization to "aColor".
    #
    # 	@param col_name the string reprsenting the color of the Element in
    #		the bridges Visualization; supported named colors are
    #  	"red", "green", "blue", "yellow", "cyan", "magenta", "white", "black",
    #  	"orange", "turquoise", "maroon", "aquamarine", "azure", "beige",
    #  	"brown", "tan", "olive", "chartreuse", "khaki", "bisque", "coral",
    #  	"pink", "lavender", "purple", "gold"
    #
    #
    def set_color(self, *args, **kwargs):
        """
        Usage: requires either 3 ints 0-255 for RGB and an optional float 0.0-1.0 for alpha or a str of a web color
        can also key the RGBA values with r, g, b, a or red, green, blue, alpha respectively and col_name for the str
        :param args: int, int, int optional float or str
        :param kwargs: r/red: int, b/blue: int, g/green: int optional a/alpha: float or col_name: str
        :return: None
        """
        self.color.set_color(*args, **kwargs)

    ##
    #
    #	Get the color of the link in the bridges Visualization
    #
    #	@return the Color object representing the color of the link
    #
    #
    def get_color(self):
        return self.color

    ##
    # Sets the opacity of the link in the bridges Visualization
    #
    # @param opacity a float between 0 and 1 representing how transparent
    #	the node
    #            should be on the bridges Visualization. 0 for invisible, 1 for
    #            fully visible, a decimal between 0 and 1 for varying
    #            transparency.
    #
    def set_opacity(self, opacity):
        self.color.set_alpha(opacity)

    ##
    #
    # 	Get the opacity of the link in the bridges Visualization
    #
    # 	@return the opacity value (in the range 0.0-1.0
    #
    #
    def get_opacity(self):
        return self.color.get_alpha()

    def get_link_properties(self):
        link_props = self.QUOTE + "color" + self.QUOTE + self.COLON + self.OPEN_BOX + str(self.get_color().get_red()) + self.COMMA + str(self.get_color().get_green()) + self.COMMA + str(self.get_color().get_blue()) + self.COMMA + str(self.get_color().get_alpha()) + self.CLOSE_BOX + self.COMMA + self.QUOTE + "thickness" + self.QUOTE + self.COLON + str(self.get_thickness()) + self.COMMA + self.QUOTE + "weight" + self.QUOTE + self.COLON + str(self.get_weight())
        return link_props

    def get_label(self):
        return self.label

    def set_label(self, label):
        self.label = label





class Book:

    def __init__(self, authorName, authorBirth, authorDeath, title, lang, genre, subject, numChars, numWords,
                 numSentences, numDifficultWords, url, downloads):
        self.authorName = authorName
        self.authorBirth = authorBirth
        self.authorDeath = authorDeath
        self.title = title
        self.lang = lang
        self.genre = genre
        self.subject = subject
        self.url = url
        self.numChars = numChars
        self.numWords = numWords
        self.numSentences = numSentences
        self.numDifficultWords = numDifficultWords

    def get_author_name(self):
        return self.authorName

    def set_author_name(self, author_name):
        self.authorName = author_name

    def get_author_birth(self):
        return self.authorBirth

    def set_author_birth(self, author_birth):
        self.authorBirth = author_birth

    def get_author_death(self):
        return self.authorDeath

    def set_author_death(self,author_death):
        self.authorDeath = author_death

    def get_title(self):
        return self.title

    def set_title(self,title):
        self.title = title

    def get_lang(self):
        return self.lang

    def set_lang(self, lang):
        self.lang = lang

    def get_genre(self):
        return self.genre

    def set_genre(self, genre):
        self.genre = genre

    def get_subject(self):
        return self.subject

    def set_subject(self, subject):
        self.subject = subject

    def get_url(self):
        return self.url

    def set_url(self,url):
        self.url = url

    def get_num_chars(self):
        return self.numChars

    def set_num_chars(self,num_chars):
        self.numChars = num_chars

    def get_num_words(self):
        return self.numWords

    def set_num_words(self,num_words):
        self.numWords = num_words

    def get_num_sentences(self):
        return self.numSentences

    def set_num_sentences(self,num_sentences):
        self.numSentences = num_sentences

    def get_num_difficult_words(self):
        return self.numDifficultWords

    def set_num_difficult_words(self, num_difficult_words):
        self.numDifficultWords = num_difficult_words




from bridges import *
import os
user = os.environ.get("BRIDGES_USER_NAME")
key = os.environ.get("BRIDGES_API_KEY")
bridges = Bridges(3100 , user, key)
bridges.connector.set_server("clone")

grid = bridges.get_color_grid_from_assignment(user, 101)
bridges.set_data_structure(grid)
bridges.visualize()

grid = bridges.get_color_grid_from_assignment(user, 100)
bridges.set_data_structure(grid)
bridges.visualize()



from bridges.data_src_dependent import data_source
from bridges.bridges import Bridges
import os
user = os.environ.get("BRIDGES_USER_NAME")
key = os.environ.get("BRIDGES_API_KEY")
bridges = Bridges(100, user, key)
bridges.connector.set_server("clone")

go = data_source.get_osm_data("charlotte")

print(go.name)
print(go.longitude_range)
print(go.latitude_range)
print(go.cartesian_range_x)
print(go.cartesian_range_y)

try:
    go.edges = [1, 2, 3]
except ValueError:
    print("edge setter works")
try:
    go.vertices = [1, 2, 3]
except ValueError:
    print("vertex setter works")

gr = go.get_graph()

bridges.set_data_structure(gr)
bridges.visualize()



#!/usr/bin/env python
from bridges.tree_element import *
##
# 	@brief This class is extended from the TreeElement class  and can be used to create
#	binary tree element objects.
#
# 	The BinTree element class is the building block for creating binary tree structures.
# 	It contains two children (viz., left, right).
#
#  BinTreeElement contains a visualizer (ElementVisualizer) object for setting visual
#  attributes (color, shape, opacity, size), necessary for displaying them in a
#  web browser.
#
#  Elements also have a LinkVisualizer object, that is used when they are linked to
#  another element, appropriate for setting link attributes, for instance, between
#  the current element and its left or  right child
#
# @author Kalpathi Subramanian, Mihai Mehedint
#
#
#
class BinTreeElement(TreeElement):
    left = object()

    # the left pointer
    right = object()


    ##
    #
    # 	Constructs an empty Binary Tree Element
    # @param e the generic object that TreeElement will hold
    # @param label the label of TreeElement that shows up on the bridges visualization
    # @param left the TreeElement to be assigned to the left pointer of this TreeElement
    # @param right the TreeElement to be assigned to the right pointer of this TreeElement
    #
    def __init__(self, label=None, e=None, left=None, right=None ):
        if label is None and e is None and left is None and right is None:
            super(BinTreeElement, self).__init__()
            super(BinTreeElement, self).add_child(left)
            super(BinTreeElement, self).add_child(right)
        elif e is not None and left is not None and right is not None:
            super(BinTreeElement, self).__init__(e = e)
            super(BinTreeElement, self).add_child(left)
            super(BinTreeElement, self).add_child(right)
        elif e is not None and label is not None and left is None and right is None:
            super(BinTreeElement, self).__init__(label = label, e = e)
            super(BinTreeElement, self).add_child(None)
            super(BinTreeElement, self).add_child(None)
        elif e is not None and label is None and left is None and right is None:
            super(BinTreeElement, self).__init__(e=e)
            super(BinTreeElement, self).add_child(None)
            super(BinTreeElement, self).add_child(None)




    ##
    #
    #	This method gets the data structure type
    #
    #	@return  The date structure type as a string
    #
    #
    def get_data_structure_type(self):
        return "BinaryTree"

    ##
    #
    # This method returns the left tree element pointer
    # @return the left child of this TreeElement
    #
    #
    def get_left(self):
        return super(BinTreeElement, self).get_child(0)

    ##
    #
    # This method sets the left tree element pointer
    # @param left the TreeElement that should be assigned to the left child
    #
    #
    def set_left(self, left):
        super(BinTreeElement, self).set_child(0, left)

    ##
    #
    # This method returns the right tree element pointer
    #
    # @return the right child of this TreeElement
    #
    #
    def get_right(self):
        return super(BinTreeElement, self).get_child(1)

    ##
    #
    # This method sets the right tree element pointer
    #
    # @param right the TreeElement that should be assigned to the right child
    #
    #
    def set_right(self, right):
        super(BinTreeElement, self).set_child(1, right)



#!/usr/bin/env python
from bridges.element import *
from bridges.link_visualizer import *


 ## @brief This class extends Element to represent general trees with
 #	arbitrary number of children.
 #
 # 	TreeElement nodes can have an arbitrary number of child nodes(held in
 # 	in a vector in the order in which they were added). The
 # 	visualization of trees assumes that the children are drawn in order
 # 	from left to right.
 #
 #  Tree Elements have labels (string) that are displayed on the visualization.
 #  Elements take an generic object E as a user defined parameter, which can be
 #  any native type or object.
 #
 #  Elements contain a visualizer (ElementVisualizer) object for setting visual
 #  attributes (color, shape, opacity, size), necessary for displaying them in a web
 #  browser.
 #
 #  Elements also have a LinkVisualizer object that is used when they are
 #  linked to another element, appropriate for setting link attributes, between parent
 # 	and child nodes.
 #
 # 	@author Matthew McQuaigue
 #
class TreeElement(Element):
    # holds all children of the node
    children = []

    QUOTE = "\""
    COMMA = ","
    COLON = ":"
    OPEN_CURLY = "{"
    CLOSE_CURLY = "}"
    OPEN_PAREN = "("
    CLOSE_PAREN = ")"
    OPEN_BOX = "["
    CLOSE_BOX = "]"

    ##
    # Constructs an TreeElement
    # set to null.
    # @param e the generic object that TreeElement will hold
    # @param label the label of TreeElement that shows up on the bridges
    # @param left the TreeElement to be assigned to the child 0
    # @param right the TreeElement to be assigned to the child 1
    #
    def __init__(self, label=None, e=None, left=None, right=None):
        if label is None and e is None and left is None and right is None:
            super(TreeElement, self).__init__()
            self.children = []
        if e is not None and label is None and left is None and right is None:
            super(TreeElement, self).__init__(val = e)
            self.children = []
        elif e is not None and left is not None and right is not None:
            super(TreeElement, self).__init__(val = e)
            self.children = []
            self.children.append(left)
            self.children.append(right)
        elif e is None and left is not None and right is not None:
            super(TreeElement, self).__init__()
            self.children = []
            self.children.append(left)
            self.children.append(right)
        elif label is not None and e is not None and left is None and right is None:
            super(TreeElement, self).__init__(label = label, val = e)
            self.children = []
        elif e is not None and label is None and left is None and right is None:
            super(TreeElement, self).__init__(val = e)
            self.children = []



    ##
    # 	This method gets the data structure type
    #
    # 	@return  The date structure type as a string
    #
    def get_data_structure_type(self):
        return "Tree"

    ##
    #   Adds a child to the node

    def add_child(self, child):
        self.children.append(child)

    ##
    #
    #  Returns the number of children at this node
    #
    #  @return  number of children

    def get_number_of_children(self):
        return len(self.children)

    ##
    # 	adds a child to the node - will be added at the next open position
    #
    #  @param  child to be added
    #
    #  @return none
    def set_child(self, index, child):
        if index < len(self.children):
            self.children[index] = child

    ##
    # 	gets a child at a particular index
    #
    #  @param  index into the list of children
    #
    #  @return child to be returned
    #
    #
    def get_child(self, index):
        if index < len(self.children):
            return self.children[index]
        else:
            return None

    ##
    # 	Get  hierarchical JSON of the tree representation
    #
    # 	@return the JSON string
    #
    def get_data_structure_representation(self):
        json_str = self.QUOTE + "nodes" + self.QUOTE + self.COLON + self.OPEN_CURLY + self.pre_order(self) + self.CLOSE_CURLY + self.CLOSE_CURLY
        return json_str

    ##
    # 	Use a preorder traversal to directly extract a hierarchical JSON
    # 	representation of the tree.
    #
    #
    def pre_order(self, root):
        k=0
        json_str = ""
        children = ""
        link_props = ""
        elem_rep = ""
        t_str = str()
        num = root.get_number_of_children()
        if root is not None:
            #  first get the node representation
            elem_rep = root.get_element_representation()
            #  remove surrounding curly braces
            t_str = elem_rep[1: len(elem_rep) - 1]
            json_str += t_str
            #  now get the children
            if root.get_number_of_children() > 0:
                json_str += self.COMMA + self.QUOTE + "children" + self.QUOTE + self.COLON + self.OPEN_BOX

            while k < root.get_number_of_children():

                if root.get_child(k) is None:
                    json_str += self.OPEN_CURLY + self.QUOTE + "name" + self.QUOTE + self.COLON + self.QUOTE + "NULL" + self.QUOTE + self.CLOSE_CURLY + self.COMMA
                else:
                    lv = LinkVisualizer()
                    lv = root.get_link_visualizer(root.get_child(k))
                    json_str += self.OPEN_CURLY
                    if lv is not None:
                        json_str += self.QUOTE + "linkProperties" + self.QUOTE + self.COLON + self.OPEN_CURLY + self.QUOTE + "color" + self.QUOTE + self.COLON + self.OPEN_BOX + str(lv.get_color().get_red()) + self.COMMA + str(lv.get_color().get_green()) + self.COMMA + str(lv.get_color().get_blue()) + self.COMMA + str(lv.get_color().get_alpha()) + self.CLOSE_BOX + self.COMMA + self.QUOTE + "thickness" + self.QUOTE + self.COLON + str(lv.get_thickness()) + self.CLOSE_CURLY + self.COMMA
                    else:
                        json_str += "linkProperties" + self.COLON + "{}" + self.COMMA
                    #  process its children
                    json_str += self.pre_order(root.get_child(k))
                    json_str += self.CLOSE_CURLY + self.COMMA
                k += 1
            #  process its children
            #  remove last comma
            if len(json_str) > 0:
                json_str = json_str[0: len(json_str) - 1]
            #  end of children
            json_str += self.CLOSE_BOX
        return json_str



#!/usr/bin/env python
from bridges.element import *


##
# @brief This class is used to represent the edges in a graph and will
# 	appear as links in the BRIDGES graph visualization.
# This object is used in graphs and graph algorithms such as DFS, BFS and shortest
# path algorithms that need to visit graph edges. The adjacency list
# representation uses them as the generic paramter, as SLelement<Edge>
# bridges represents Edges as links between pairs of elements
#
# @author K.R. Subramanian
#
#
class Edge(object):
    weight = int()
    # vertex = Element()

    #  refers to a terminating vertex
    edge_data = str()

    ##
    #
    # Constructors
    # @param wt integer, representing  edge weight
    # @param v the terminating vertex of the edge
    #
    def __init__(self, wt=None, v=None):
        if wt is not None:
            self.weight = wt
        else:
            self.weight = 0
        if v is not None:
            self.vertex = v
        self.edge_data = ""

    ##
    #
    # Set edge weight to "wt"
    #
    # @param wt  -  graph edge weight
    #
    #
    def set_weight(self, wt):
        self.weight = wt

    ##
    #
    # Get edge weight
    #
    # @return the weight of edge
    #
    #
    def get_weight(self):
        return self.weight

    ##
    #
    # Set terminating Element of the edge
    #
    # @param v the identifier of the terminating Element
    #
    #
    def set_vertex(self, v):
        self.vertex = v

    ##
    #
    # Get identifer of the terminating Element of edge
    #
    # @return the string identifier of the terminating Element
    #
    #
    def get_vertex(self):
        return self.vertex

    ##
    #
    # Set edge to weight  of "wt" and terminating Elememt of "v".
    #
    # @param wt edge weight
    # @param v the identifier of the terminating Element
    #
    #
    def set_edge(self, wt, v):
        self.weight = wt
        self.vertex = v

    ##
    # Set Edge data (represented as a string for now)
    #
    # @param string: application data
    #
    def set_edge_data(self, data):
        self.edge_data = data

    ##
    # Get edge data
    #
    # @return the edge data
    #
    def get_edge_data(self):
        return self.edge_data

    ##
    # Returns this edge
    #
    def get_edge(self):
        return self



from bridges import *
from bridges.kd_tree_element import *

bridges = Bridges(0, "test", "211416381091")
bridges.connector.set_server("local")
bridges.json_flag = True

bridges.set_title("a KD Tree Example")

t0 = KDTreeElement(key = 50, dim = 0)
t1 = KDTreeElement(key = 25, dim = 1)
t2 = KDTreeElement(key = 75, dim = 1)
t3 = KDTreeElement(key = 20, dim = 0)
t4 = KDTreeElement(key = 30, dim = 0)
t5 = KDTreeElement(key = 60, dim = 0)
t6 = KDTreeElement(key = 80, dim = 0)

t0.set_left(t1)
t0.set_right(t2)
t1.set_left(t3)
t1.set_label("X")
t1.set_right(t4)
t2.set_left(t5)
t2.set_left(t6)

t0.set_label("X")
t1.set_label("Y")
t2.set_label("Y")
t3.set_label("X")
t4.set_label("X")
t5.set_label("X")
t6.set_label("X")

t0.get_visualizer().set_color("red")

t1.get_visualizer().set_color("orange")
t4.get_visualizer().set_color("orange")
t5.get_visualizer().set_color("orange")
t6.get_visualizer().set_color("orange")

bridges.set_data_structure(t0)
bridges.visualize()


from bridges.connector import *
from bridges import ColorGrid
import json
##
# 	@brief The bridges class is the main class that provides interfaces to datasets,
#	maintains user and assignment information, and connects to the bridges server.
#
#  	The bridges class is responsible  for initializing the bridges system, specifying
#  	parameters (user id, assignment id, title, description, data structure
# 	type, etc) for the student assignment, generating the data structure representation
# 	and transmission to the bridges server. In addition, it provides interfaces to
# 	a number of real-world datasets, that makes it easy to access the data for use
#  	algorithms/data structure assignments. <br>
#
#   <b>Datasets.</b> The datasets that are currently supported through the BRIDGES API
# 	include USGS Earthquake Data, IMDB Actor/Movie Data (2 versions), Gutenberg Book
# 	Collection Meta Data, a Video Game Dataset and Shakespeare Dataset. More information
# 	is found in the respective methods (below) and at <p>
# 	http://bridgesuncc.github.io/datasets.html <p>
#
# 	A typical bridges program includes creating the bridges object, followed by creation
#   of the data structure by the user, assigning visual attributes to elements of the
# 	data structure, followed by specification of teh data structure type  and the
# 	call to visualize the data structure (bridges::setDataStructure() and visualize()
# 	methods).
#
#  	@author Sean Gallagher, Kalpathi Subramanaian, Mihai Mehedint, David Burlinson.
#
#


class Bridges:
    vis_type = ""
    ds_handle = None
    title = str()
    description = str()
    coord_system_type = "cartesian"
    map_overlay = False
    username = ""
    assignment = int()
    assignment_part = int()
    _MaxTitleSize = 50
    _MaxDescSize = 250
    json_flag = False

    projection_options = {"cartesian", "albersusa", "equirectangular"}

    QUOTE = "\""
    COMMA = ","
    COLON = ":"
    OPEN_CURLY = "{"
    CLOSE_CURLY = "}"
    OPEN_PAREN = "("
    CLOSE_PAREN = ")"
    OPEN_BOX = "["
    CLOSE_BOX = "]"

    ##
    # Initialize bridges (Constructor)
    # @param assignment this is the assignmen id (integer)
    # @param appl_id    this is the appl authentication key(from the bridges account)
    # @param username   this is the username (from the bridges account)
    #
    def __init__(self, assignment, username, appl_id):
        self.assignment_part = 0
        self.title = str()
        self.description = str()
        self.set_assignment(assignment)
        self.key = appl_id
        self.connector = Connector(appl_id, username, assignment)
        self.username = username
        self.coord_system_type = "cartesian"

    ##
    #
    #  	This method sets  the handle to the current data structure; this can
    # 	be an array, the head of a linked list, root of a tree structure, a graph
    # 	Arrays of upto 3 dimensions are suppported. It can be any of the data
    # 	structures supported by BRIDGES. Polymorphism and type casting is used
    # 	to determine the actual data structure and extract its representtion.
    #
    #  @param ds   The data structure to set (any of the subclasses of DataStruct)
    #
    #
    def set_data_structure(self, ds):
        try:
            self.ds_handle = ds
            self.vis_type = ds.get_data_structure_type()
        except ValueError:
            print("Exception Thrown: Data structure passed to BRIDGES is null!\n")

    def set_visualize_JSON(self, flag):
        self.json_flag = flag

    ##
    #
    #  This method generates the representation of the current data structure (JSON)
    #  and sends that to the bridges server for generating a visualization.
    #
    def visualize(self):
        nodes_links_str = ""

        if (self.vis_type == "Tree" or self.vis_type == "BinaryTree" or self.vis_type == "SinglyLinkedList", self.vis_type == "DoublyLinkedList", self.vis_type == "MultiList", self.vis_type == "CircularSinglyLinkedList", self.vis_type == "CircularDoublyLinkedList", self.vis_type == "Array", self.vis_type == "GraphAdjacencyList", self.vis_type == "ColorGrid", self.vis_type == "KDTree", self.vis_type == "SymbolCollection"):
            nodes_links_str = self.ds_handle.get_data_structure_representation()

        ds = {
            "visual": self.vis_type,
            "title": self.title,
            "description": self.description,
            "coord_system_type": self.coord_system_type,
            "map_overlay": self.map_overlay,
        }
        ds_json = json.dumps(ds)[:-1] + ", "

        if self.vis_type == "Array":
            dims = [1,1,1]
            ds_array = self.ds_handle
            num_dims = ds_array.get_num_dimensions()
            ds_array.get_dimensions(dims)
            ds_json += self.QUOTE + "dims" + self.QUOTE + self.COLON + self.OPEN_BOX + str(dims[0]) + self.COMMA + str(dims[1]) + self.COMMA + str(dims[2]) + self.CLOSE_BOX + self.COMMA

            ds_json += nodes_links_str
        else:
            ds_json += nodes_links_str

        if self.json_flag:
            print(ds_json)

        response = self.connector.post("/assignments/" + self.get_assignment(), ds_json)

        if response == 200:
            print(
                "\nCheck Your Visualization at the following link:\n\n" + self.connector.get_server_url() + "/assignments/" + str(self.assignment) + "/" + self.username + "\n\n")

            self.assignment_part = self.assignment_part + 1

    ##
    # 	set the assignment id
    #
    #  @param assignment number
    #
    #
    def set_assignment(self, assignment):
        if assignment < 0:
            ValueError("Assignment value must be >= 0")
        elif self.assignment >= 0:
            self.assignment_part = 0
        self.assignment = assignment

    ##
    # 	Get the assignment id
    #
    #   @return assignment as a string
    #
    #
    def get_assignment(self):
        if self.assignment_part < 10:
            return str(self.assignment) + ".0" + str(self.assignment_part)
        else:
            return str(self.assignment) + "." + str(self.assignment_part)

    ##
    #
    #  @param title title used in the visualization;
    #
    #
    def set_title(self, title):
        if len(title) > self._MaxTitleSize:
            print(
                "Visualization Title restricted to" + str(self._MaxTitleSize) + " characters." + " truncated title...")
            self.title = title[:self._MaxTitleSize]
        else:
            self.title = title

    ##
    #
    #  @param description description to annotate the visualization;
    #
    #
    def set_description(self, description):
        if len(description) > self._MaxDescSize:
            print("Visualization Description restricted to " + str(self._MaxDescSize) + " Truncating description..")
            self.description = description[0:self._MaxDescSize]
        else:
            self.description = description

    def set_map_overlay(self, flag):
        self.map_overlay = flag

    def set_coord_system_type(self, coord):
        if coord in self.projection_options:
            self.coord_system_type = coord
        else:
            print("Unrecognized coordinate system \'" + coord + "\', defaulting to cartesian. Options:")
            self.coord_system_type = "cartesian"

    def get_color_grid_from_assignment(self, user: str, assignment: int, subassignment: int = 0) -> ColorGrid:
        """
        Reconstruct a ColorGrid from an existing ColorGrid on the bridges server

        :param str user: the name of the user who uploaded the assignment
        :param int assignment: the ID of the assignment to get
        :param int subassignment: the ID of the subassignment to get (default 0)
        :return: ColorGrid: the ColorGrid stored in the bridges server
        """
        from bridges.data_src_dependent.data_source import get_color_grid_from_assignment
        return get_color_grid_from_assignment(self.connector.server_url, user, assignment, subassignment)

    def get_username(self):
        return self.username.replace(" ", "+")

    def get_assignment_id(self):
        return self.assignment





import json


class SymbolCollection:

    def __init__(self):
        self.symbols = dict()
        self.domain = 100.0

    def get_data_structure_type(self):
        return "SymbolCollection"

    def add_symbol(self, s):
        self.symbols[s.get_identifier()] = s

    def update_axis_domains(self, s):
        dims = s.get_dimensions()

        if abs(dims[0]) > self.domain:
            self.domain = abs(dims[0])
        if abs(dims[1]) > self.domain:
            self.domain = abs(dims[1])

        if abs(dims[2]) > self.domain:
            self.domain = abs(dims[2])
        if abs(dims[3]) > self.domain:
            self.domain = abs(dims[3])

    def get_data_structure_representation(self):
        symbol_json = []

        for key in self.symbols.keys():
            self.update_axis_domains(self.symbols[key])
            symbol_json.append(self.symbols[key].get_json_representation())

        symbol_json = json.dumps(symbol_json)
        return "\"""domainX\""":[" + str(-self.domain) + "," + str(self.domain) + "],\"""domainY\""":[" + str(-self.domain) + "," + str(self.domain) + "], " + "\"""symbols\""":" + symbol_json + "}"




class Shakespeare:

    def __init__(self, title: str = "", type: str = "", text: str = ""):
        self.title = title
        self.type = type
        self.text = text

    def get_title(self):
        return self.title

    def set_title(self, title):
        self.title = title

    def get_type(self):
        return self.type

    def set_type(self,type):
        self.type = type

    def get_text(self):
        return self.text

    def set_text(self, text):
        self.text = text



from bridges.bin_tree_element import *
##
#    @brief The BSTElement class is the building block for creating binary search trees.
#
#   The BSTElement class is the building block for creating binary search tree structures.
#    It contains two children (viz., left, right), and a search key, to be used
#    in search operations .
#
#    BSTElement contains a visualizer (ElementVisualizer) object for setting visual
#    attributes (color, shape, opacity, size), necessary for displaying them in a
#    web browser.
#
#    BST Elements also have a LinkVisualizer object, that is used when they are linked to
#    another element, appropriate for setting link attributes, for instance, between
#    the current element and its left or right child
#
#  	@author Kalpathi Subramanian, Mihai Mehedint
#
#  	@date 6/22/16, 1/7/17, 5/17/17
#
#  	@brief This class extends the BinTreeElement class by adding a 'key' value
#  	for use in a binary search tree implementations.
#
#
class BSTElement(BinTreeElement):
    key = object()

    ##
    #
    #  Construct an empty BSTElement with no key assigned and left and
    #  right pointers set to null.
    #  @param key the key to be used in a binary search tree implementation
	#  @param e the object this BSTElement is holding
	#  @param left the BSTElement that should be assigned to the left pointer
	#  @param right the BSTElement that should be assigned to the right pointer
    #
    def __init__(self, key = None, e = None, left = None, right = None, label = None):
        if key is not None:
            self.set_key(key)
        if e is None and left is None and right is None:
            super(BSTElement, self).__init__()
        elif e is None and left is not None and right is not None:
            super(BSTElement, self).__init__(left = left, right = right)
        elif e is not None and label is not None:
            super(BSTElement, self).__init__(label = label, e = e)
        elif e is not None and left is None and right is None:
            super(BSTElement, self).__init__(e = e)
        elif e is not None and left is not None and right is not None:
            super(BSTElement, self).__init__(e = e, left = left, right = right)





    ##
    # 	This method gets the data structure type
    #
    # 	@return  The date structure type as a string
    #
    def get_data_structure_type(self):
        return "BinarySearchTree"


    ##
    # 	Return the key of the BSTElement
    #
    # 	@return the key of this BSTElement
    #
    def get_key(self):
        return self.key

    ##
    #
    # 	Set the key of the BSTElement to key
    #  	@param key the key to set
    #
    def set_key(self, key):
        self.key = key
        #  add this to the element's properties

    ##
    # Return the left child of the  BSTElement
    # @return the left child of this BSTElement
    def get_left(self):
        return super(BSTElement, self).get_left()

    ##
    # Return the right child of the BSTElement
    # @return the right child of this BSTElement
    def get_right(self):
        return super(BSTElement, self).get_right()




from bridges import *
import os
user = os.environ.get("BRIDGES_USER_NAME")
key = os.environ.get("BRIDGES_API_KEY")
bridges = Bridges(100, user, key)
bridges.connector.set_server("clone")

len_x = 256
len_y = 256

color_grid = ColorGrid(len_x, len_y)

for x in range(0, len_x):
    for y in range(0, len_y):
        color = Color(y, y, y, 1.0)
        color_grid.set(x, y, color)

bridges.set_data_structure(color_grid)
bridges.visualize()



from bridges.element import *
import traceback
##
#
#  @brief The GraphAdjMatrix class can be used to represent adjacency matrix based
#      graphs in BRIDGES
#
#  The GraphAdjMatrix class can be used to represent adjacency matrix based  graphs
#  in BRIDGES; it takes 2 generic parameters: (1) K, which is an orderable
#  key value used in accessing vertices (in constant time) using a hashmap. This
#  permits data sets that need to be accessed by keys that are strings, and
#  (2) E, an application defined type, and used in the Edge representation.
#  The class is simply a wrapper  around the Java Hashmap class
#  and, thus, derives all its operations from it.
#  BRIDGES provides methods to visualize the graph  and its contents.
#
#  The vertices of the graph are held in a Java hashmap, for near constant time access;
#  this lets us use strings or integral ids for vertices. The edges are accessed
#	by a second hashmap from each vertex, again assuring near constant access time.
#  Each edge contains the terminating vertex id and weight, as defined by  the Edge
#	class structure.
#
#  Convenience methods are provided to add vertices and edges to the graph. Edges
#  are retrieved by using the dual hashmap, given the vertex ids of the edge.
#
#  @author Kalpathi Subramanian, Mihai Mehedint
#
#
#  \sa Example tutorial at <p>
#      ?? TO DO
#
#
class GraphAdjMatrix():
    #graph vertices list
    # vertices = dict()

    #holds the adjacency list of edges
    # matrix = dict()

    #holds edge information for graph edges
    # edge_data = dict()

    QUOTE = "\""
    COMMA = ","
    COLON = ":"
    OPEN_CURLY = "{"
    CLOSE_CURLY = "}"
    OPEN_PAREN = "("
    CLOSE_PAREN = ")"
    OPEN_BOX = "["
    CLOSE_BOX = "]"

    ##
    # 	Constructor
    #
    def __init__(self, size = None):
        self.vertices = dict()
        self.matrix = dict()
        self.edge_data = dict()

    ##
    #
    #	This method gets the data structure type
    #
    #	@return  The date structure type as a string
    #
    #
    def get_data_structure_type(self):
        return "GraphAdjacencyMatrix"

    ##
    # 	Adds a new vertex to the graph, initializes the  adjacency
    # 	list; user is responsible for checking if the vertex already
    # 	exists. This method will replace the value for this key
    #
    # 	@param k - vertex key value
    # 	@param 3 - user specified data, part of the vertex data
    #
    #
    def add_vertex(self, k, e):
        #  note: it is the user's responsibility to  check
        #  for duplicate vertices
        self.vertices[k] = Element(val = e)
        self.vertices.get(k).set_label(str(k))

        #  create a hashmap for this vertex
        self.matrix[k] =  dict()
        #  fill up this vertex's row and column elements
        for k, element in self.vertices.items():
            (self.matrix.get(k))[element.get_label()] = 0 #  row
            (self.matrix.get(element.get_label()))[k] =  0 #  col

    ##
    #	Adds a new edge to the graph, adds it to the index corresponding to
    #	the source, destination vertex ids;  this version of the method assumes
    #	an edge weight of 1 (unweighted graph); user is responsible for checking if the
    #	vertices already exist, else an exception is thrown.
    #
    #	@param src - source vertex of edge
    #	@param dest - destination  vertex of edge
    #
    #
    def add_edge(self, src, dest, weight = None):
        #  check to see if the two vertices exist, else
        #  throw an exception
        try:
            if self.vertices.get(src) is None or self.vertices.get(dest) is None:
                raise ValueError("Vertex " + src + " or " + dest + " does not exist! Add the vertex before creating the edge.")
        except Exception as e:
            traceback.print_tb(e.__traceback__)
        if weight is not None:
            (self.matrix.get(src))[dest] = weight
        else:
            (self.matrix.get(src))[dest] =  1


    ##
    #
    #	This method returns the graph nodes
    #
    #	return -- vertices held in an unordered  map
    #
    #
    def get_vertices(self):
        return self.vertices

    ##
    #
    #	Gets the adjacency matrix
    #
    #	@return - the graph's adjacency matrix
    #
    def get_adjacency_matrix(self, key = None):
        if key is None:
            return self.matrix
        else:
            return self.matrix.get(key)

    ##
    #
    #	 This is a convenience method to simplify access to the link visualizer;
    #	 the method assumes the vertex names point to existing vertices, else an exception
    #	 is thrown
    #
    #
    def get_link_visualizer(self, src, dest):
        #  get the source and destination vertex elements
        #  and check to see if they exist
        v1 = self.vertices.get(src)
        v2 = self.vertices.get(dest)
        try:
            if v1 is None or v2 is None:
                raise ValueError("Vertex " + src + " or " + dest + " does not exist! First add the vertices to the graph.")
        except Exception as e:
            traceback.print_tb(e.__traceback__)
        return v1.get_link_visualizer(v2)

    ##
    #
    #	This is a convenience method to simplify access to the element visualizer;
    #	the method assumes the vertex name points to an existing vertice, else an
    #	exception is thrown
    #
    #
    def get_visualizer(self, vertex):
        #  get the source and destination vertex elements
        #  and check to see if they exist
        v = self.vertices.get(vertex)
        try:
            if v is None:
                raise ValueError("Vertex " + vertex + " does not exist! First add the vertices to the graph.")
        except Exception as e:
            traceback.print_tb(e.__traceback__)
        return v.get_visualizer()

    ##
    # 	Get the JSON representation of the the data structure
    #
    def get_data_structure_representation(self):
        #  map to reorder the nodes for building JSON
        node_map = dict()
        #  get teh list nodes
        nodes = []

        for key, value in self.vertices.items():
            nodes.append(value)

        #  remap  map these nodes to  0...MaxNodes-1
        #  and build the nodes JSON
        nodes_JSON = str()
        k = 0
        while k < len(nodes):
            node_map[nodes[k]] = k
            nodes_JSON+=(nodes[k].get_element_representation())
            nodes_JSON+=(self.COMMA)
            k += 1

        #  remove the last comma
        if len(nodes) != 0:
            nodes_JSON = nodes_JSON[:-1]
        #  build the links JSON - traverse the adj. lists

        links_JSON = str()
        for el_src in self.matrix.items():
            src_vert = self.vertices.get(el_src[0])
            src_indx = node_map.get(src_vert)
            for el_dest in el_src[1].items():
                dest_vert = self.vertices.get(el_dest[0])
                if el_dest[1] > 0:
                    dest_indx = node_map.get(dest_vert)
                    links_JSON+=(src_vert.get_link_representation(src_vert.get_link_visualizer(dest_vert), str(src_indx), str(dest_indx)))
                    links_JSON+=self.COMMA
        #  remove the last comma
        if len(links_JSON) > 0:
            links_JSON = links_JSON[:-1]
        json_str = self.QUOTE + "nodes" + self.QUOTE + self.COLON + self.OPEN_BOX + nodes_JSON.__str__() + self.CLOSE_BOX + self.COMMA + self.QUOTE + "links" + self.QUOTE + self.COLON + self.OPEN_BOX + links_JSON.__str__() + self.CLOSE_BOX + self.CLOSE_CURLY
        return json_str




class Student:


    def __init__(self, id, name, study, email, likeColor, dislikeColor, creditHours):
        self.id = id
        self.name = name
        self.study = study
        self.email = email
        self.likeColor = likeColor
        self.dislikeColor = dislikeColor
        self.creditHours = creditHours

    def getid(self):
        return self.id
    def getName(self):
        return self.name
    def getStudy(self):
        return self.study
    def getEmail(self):
        return self.email
    def getLikeColor(self):
        return self.likeColor
    def getDislikeColor(self):
        return self.dislikeColor
    def getCreditHours(self):
        return self.creditHours


import json
from bridges.symbol import *

class Circle(Symbol):

    def __init__(self, locx = None, locy = None, r = None):
        super(Circle, self).__init__()
        if r is not None:
            if r < 0:
                raise ValueError("Radius value needs to be positive")
            self.radius = r
        else:
            self.radius = 10

        if locx is not None and locy is not None:
            self.set_location(locx, locy)

    def get_name(self):
        return "circle"

    def set_radius(self,r):
        if r < 0:
            raise ValueError("Illegal value for radius. Must be positive")
        self.radius = r

    def set_circle(self, locx, locy, r):
        self.set_location(locx, locy)
        if r < 0:
            raise ValueError("Radius value needs to be positive")
        self.radius = r

    def get_dimensions(self):
        dims = []
        location = self.get_location()

        dims.append(location[0] - self.radius)
        dims.append(location[0] + self.radius)
        dims.append(location[1] - self.radius)
        dims.append(location[1] + self.radius)

        return dims

    def get_json_representation(self):

        ds_json = super(Circle, self).get_json_representation()
        ds_json["name"] = self.get_label()
        ds_json["shape"] = self.get_name()
        ds_json["r"] = self.radius

        return ds_json


#!/usr/bin/env python
from bridges.element import *

##
# @brief This class can be used to create arrays of type Element<E>.
#
# @author 	Kalpathi Subramanian
#
# @date  	10/8/16, 5/17/17
#
#	This class can be used to create arrays of type Element<E>  where E
#	is a generic object representing application specific data.
#
#	Arrays are internally represented as 1D arrays; currently 1D, 2D  and
#	3D arrays are supported.
#
#
#
class Array():
    QUOTE = "\""
    COMMA = ","
    COLON = ":"
    OPEN_CURLY = "{"
    CLOSE_CURLY = "}"
    OPEN_PAREN = "("
    CLOSE_PAREN = ")"
    OPEN_BOX = "["
    CLOSE_BOX = "]"

    #  only 2D and 3D arrays supported
    dims = [1, 1, 1]

    # array_data=[]



    ##
    # Construct an array object
    # @param num_dims number of dimensions of the array
    # @param dims size of each dimension
    # @param num_elements in the array
    # @param x_dim number of elements along dimension 1
    # @param y_dim number of elements along dimension 1
    # @param z_dim number of elements along dimension 1
    #
    def __init__(self, num_dims = None, dims = None, num_elements = None, x_dim = None, y_dim = None, z_dim = None):
        if num_dims is None and dims is None and num_elements is None and x_dim is None and y_dim is None and z_dim is None:
            self.array_data = []
            self.num_dims = 1
            self.dims[0] = self.dims[1] = self.dims[2] = self.size = 0
        elif num_dims is not None and dims is not None:
            self.array_data = []
            self.set_num_dimensions(num_dims)
            self.set_dimensions(dims)
        elif num_elements is not None:
            self.array_data = []
            self.set_num_dimensions(1)
            self.dims[0] = num_elements
            self.dims[1] = self.dims[2] = 1
            self.set_dimensions(self.dims)
        elif x_dim is not None and y_dim is not None and z_dim is not None:
            self.array_data = []
            self.set_num_dimensions(3)
            self.dims[0] = x_dim
            self.dims[1] = y_dim
            self.dims[2] = z_dim
            self.set_dimensions(self.dims)
        elif x_dim is not None and y_dim is not None and z_dim is None:
            self.array_data = []
            self.set_num_dimensions(2)
            self.dims[0] = x_dim
            self.dims[1] = y_dim
            self.dims[2] = 1
            self.set_dimensions(self.dims)



    ##
    #	This method gets the data structure type
    #
    #	@return  The date structure type as a string
    #
    def get_data_structure_type(self):
        if (self.num_dims >= 1) and (self.num_dims <= 3):
            return "Array"
        else:
            raise ValueError("Invalid number of dimensions. Only 1D, 2D and 3D arrays supported at this time")

    ##
    #
    #	Set the number of dimensions of the array;
    #
    #	@param nd  number of dimensions
    #
    def set_num_dimensions(self, nd):
        if nd > 3:
            raise ValueError("Invalid number of dimensions. Only 1D, 2D and 3D arrays supported at this time")
        self.num_dims = nd

    ##
    #
    #	Get the number of dimensions of the array;
    #
    #	@return   number of dimensions
    #
    def get_num_dimensions(self):
        if self.num_dims > 3:
            raise ValueError("Invalid number of dimensions. Only 1D, 2D and 3D  arrays supported at this time")
        return self.num_dims

    ##
    #
    #	Set the size of each dimensions; also allocates  array space
    #
    #	@param dim[]  size of each dimension
    #
    def set_dimensions(self, dim):
        sz = 1
        k = 0
        while k < self.num_dims:
            Array.dims[k] = dim[k]
            sz = sz * dim[k]
            k += 1
        #  first check the dimensions are all positive
        if sz < 0:
            raise ValueError("Invalid dimension value, must be  positive")
        self.size = sz
        #  allocate space for the array
        k = 0
        while k < self.size:
            self.array_data.append(Element())
            k += 1

    ##
    #
    #	Get the size of each dimensions;
    #
    #	@param dims[]  size of each dimension is returned
    #
    def get_dimensions(self, dims):
        dims[0] = self.dims[0]
        dims[1] = self.dims[1]
        dims[2] = self.dims[2]


    ##
    #
    #	Get the array size
    #
    #	@return size
    #
    def get_size(self):
        return self.size

    def get_element(self, indx = None, x= None, y = None, z= None):
        if indx is not None:
            return self.array_data[indx]
        elif x is not None and y is not None and z is None:
            return self.array_data[y*self.dims[1]+ x]
        elif x is not None and y is not None and z is not None:
            return self.array_data[z*self.dims[0]*self.dims[1] + y*self.dims[0] + x]

    ##
    # @param x  column index into the array
    # @param y  row index into the array
    # @param z  depth index into the array
    # @param el  element object to be assigned at 'indx'
	#
    def set_element(self, indx = None, el = None, x = None, y = None, z = None):
        if indx is not None and el is not None:
            self.array_data[indx] = el
        elif el is not None and x is not None and y is not None and z is not None:
            self.array_data[z * self.dims[0] * self.dims[1] + y * self.dims[0] + x] = el
        elif el is not None and x is not None and y is not None and z is None:
            self.array_data[y * self.dims[0] + x] = el


    ##
    #
    #	Get the object at 'indx'
    #
    #	@param indx  index into the array
    #	@return Element<E>  object at 'indx'
    #
    def get_value(self, indx = None, col = None, row = None, slice = None):
        if col is not None and row is not None and slice is None:
            return self.array_data[row * self.dims[0] + col]
        if col is not None and row is not None and slice is not None:
            return self.array_data[slice * self.dims[0] * self.dims[1] + row * self.dims[0] + col]
        if col is None and row is None and slice is None and indx is not None:
            return self.array_data[indx]

    ##
    #
    #	Set the input object at 'indx'
    #
    #	@param indx  index into the array
    #	@param el  element object to be assigned at 'indx'
    #
    #
    #
    def set_value(self, indx = None, col = None, row = None, slice = None, el = None):
        if indx is not None and el is not None:
            self.array_data[indx] = el
        if col is not None and row is not None and el is not None and indx is None:
            self.array_data[row * self.dims[0] + col] = el
        if col is not None and row is not None and el is not None and slice is not None and indx is None:
            self.array_data[slice* self.dims[0] * self.dims[1] + row * self.dims[0] + col] = el


    ##
    # Generating the JSON string for a bridges array object (Array<E>[])
    #
    # @param bridges Array object
    #
    # @return JSON string
    #
    def get_data_structure_representation(self):
        nodes_JSON = str()
        links_JSON = str()

        i = 0
        while i < self.size:
            if self.array_data[i] is not None:
                nodes_JSON += (self.array_data[i].get_element_representation() + ",")
            i += 1
        #  remove last comma
        nodes_JSON = nodes_JSON[:-1]
        json_str = self.QUOTE + "nodes" + self.QUOTE + self.COLON + self.OPEN_BOX
        json_str += nodes_JSON
        json_str += self.CLOSE_BOX + self.CLOSE_CURLY
        return json_str



import json
from bridges.color import *

class Symbol:

    ids = 0

    def __init__(self):
        self.identifier = str(Symbol.ids)
        self.label = ""
        self.fill_color = Color("blue")
        self.stroke_color = Color("white")
        self.opacity = 1.0
        self.stroke_width = 1.0
        self.stroke_dash = 1
        self.location_y = 0.0
        self.location_x = 0.0
        Symbol.ids += 1

    def get_data_structure_type(self):
        return "Symbol"

    def set_label(self, label):
        self.label = label

    def get_label(self):
        return self.label

    def get_identifier(self):
        return self.identifier

    def set_fill_color(self, c):
        self.fill_color = c

    def get_fill_color(self):
        return self.fill_color

    def set_stroke_color(self,c):
        self.stroke_color = c

    def get_stroke_color(self):
        return self.stroke_color

    def set_stroke_width(self, stroke_width):
        if stroke_width <= 0.0 or stroke_width > 10.0:
            raise ValueError("Stroke width must be between 0 and 10")
        else:
            self.stroke_width = stroke_width

    def get_stroke_width(self):
        return self.stroke_width

    def set_opacity(self, o):
        if o <= 0.0 or o > 1.0:
            raise ValueError("Opacity must be between 0.0 and 1.0")
        else:
            self.opacity = 0

    def get_opacity(self):
        return self.opacity

    def set_stroke_dash(self, dash):
        if dash < 0 or dash > 10:
            raise ValueError("Dash must be between 0 and 10 (inclusive)")
        else:
            self.stroke_dash = dash

    def get_stroke_dash(self):
        return self.stroke_dash

    def set_location(self, x, y):
        float(x)
        float(y)
        if x > float('-inf') and x < float('inf') and y > float('-inf') and y < float('inf'):
            self.location_x = x
            self.location_y = y
        else:
            raise ValueError("Coordinates must be real numbers")
    def get_location(self):
        return [self.location_x, self.location_y]

    def get_dimensions(self):
        return [0.0, 0.0, 0.0, 0.0]

    def get_json_representation(self):

        ds = {
            "fill": [self.fill_color.get_red(), self.fill_color.get_green(), self.fill_color.get_blue(), self.fill_color.get_alpha()],
            "opacity": self.opacity,
            "stroke": [self.stroke_color.get_red(), self.stroke_color.get_green(), self.stroke_color.get_blue(), self.stroke_color.get_alpha()],
            "stroke-width": self.stroke_width,
            "stroke-dasharray": self.stroke_dash,
            "location": {
                "x": self.location_x,
                "y": self.location_y
            }
        }
        return ds




from bridges.data_src_dependent import data_source
from bridges.bridges import Bridges
import os
#from heapq import heappush, heappop
import queue as Q
    
def getClosest(gr, lat, lon):
    closest = -1
    dist = float('inf')

    for k in gr.vertices:
        theosmvertex = gr.get_vertex(k).get_value()
        vlat = theosmvertex.latitude
        vlon = theosmvertex.longitude
        vdist = (vlat-lat)*(vlat-lat)+(vlon-lon)*(vlon-lon)
        if (vdist < dist):
            dist = vdist
            closest = k
        
    return closest

def style_root(gr, root) :
    elvis = gr.get_visualizer(root)
    elvis.set_color(0,0,0)

def dijkstra (gr, root) :
    distance = {}
    parent = {}
    
    for v in gr.vertices:
        distance[v] = float('inf')

    distance[root] = 0

    qu = Q.PriorityQueue()
    qu.put((distance[root], root))
    

    while not qu.empty():
        (d,v) = qu.get()
#        print (str(d)+" "+str(v))
        if d > distance[v]:
            continue

        nei_link = gr.get_adjacency_list(v)
        while (nei_link != None):
            nei=nei_link.get_value().get_vertex()
            edge_length=nei_link.get_value().get_weight()

#            print ("  "+str(nei)+" "+str(edge_length))

            if distance[v]+edge_length < distance[nei]:
                distance[nei] = distance[v]+edge_length
                qu.put((distance[nei], nei))
                parent[nei] = v
                
            nei_link=nei_link.get_next()
            
    return (distance, parent)
    

def style_distance(gr, distance):
    maxd = 0
    for v in gr.vertices:
        if distance[v] != float('inf') and distance[v] > maxd:
            maxd = distance[v]

    for v in gr.vertices:
        if distance[v] != float('inf'):
            grays = (maxd-distance[v])/maxd*255
            gr.get_visualizer(v).set_color(grays, grays, grays)


def style_parent(gr, parent, dest):
    # style out all vertices
    for v in gr.vertices:
        gr.get_visualizer(v).set_color(0, 0, 0, 0.2)

    # style out all edges
    for v in gr.vertices:
        nei_link = gr.get_adjacency_list(v)

        while (nei_link != None):
            nei=nei_link.get_value().get_vertex()
            gr.get_link_visualizer(v, nei).set_color(0, 0, 0, 0.2)
            
            nei_link=nei_link.get_next()

    # Style the path between parent and dest
    prev = dest
    cur = dest
    while cur != None:
        #style node
        gr.get_visualizer(cur).set_color(0, 0, 0, 1)

        #style edge along the path
        if prev != cur:
            gr.get_link_visualizer(cur, prev).set_color(0, 0, 0, 1) 
            gr.get_link_visualizer(prev, cur).set_color(0, 0, 0, 1) #some of these back edges should not exist but bridges gives me whatever I ask
        
        prev = cur
        cur = parent.get(cur)
    
    return
            
def main():
    user = os.environ.get("BRIDGES_USER_NAME")
    key = os.environ.get("BRIDGES_API_KEY")
    bridges = Bridges(100, user, key)
    bridges.connector.set_server("clone")
    
    osm_data = data_source.get_osm_data("uncc_campus")
    
    print(osm_data.longitude_range)
    print(osm_data.latitude_range)
    
    gr = osm_data.get_graph()
    
    root = getClosest(gr,
                         (osm_data.latitude_range[0]+osm_data.latitude_range[1])/2,
                         (osm_data.longitude_range[0]+osm_data.longitude_range[1])/2)

    print (root)

    style_root(gr, root)

    (distance,parent) = dijkstra(gr,root)

    style_distance(gr, distance)
    
    bridges.set_data_structure(gr)
    bridges.visualize()

    dest = getClosest(gr,
                      (osm_data.latitude_range[0]+(osm_data.latitude_range[1]-osm_data.latitude_range[0])/4),
                      (osm_data.longitude_range[0]+(osm_data.longitude_range[1]-osm_data.longitude_range[0])/4))

    style_parent(gr,parent, dest)

    bridges.set_data_structure(gr)
    bridges.set_visualize_JSON(True)
    bridges.visualize()


if __name__ == "__main__":
    main()
